package com.ibm.sysmgt.raidmgr.dataproc.config.external;

// ---------------------------------------------------------------------
//  Copyright:    (C) International Business Machines Corporation, 2002
//
//  Project Des:  Java Common Raid Manager
//
//  Module Name:  FibreHardDrive.java
//
//  Description:
//
//  Revision(s):  Title - Description            Author          Date
//  ------------  ------------------------     -----------    ----------
//  0.0.0         Creation                      DAK            02/07/02

import java.util.*;
import com.ibm.sysmgt.raidmgr.util.*;
import com.ibm.sysmgt.raidmgr.dataproc.util.*;
import com.ibm.sysmgt.raidmgr.dataproc.config.*;

/** The Hard Drive with a Fibre Channel Interface. */
public class FibreHardDrive extends StorageDevice implements Cloneable
{
    static final long serialVersionUID = 6052677538542343484L;
    public static final int MAX_PORTS = 2;

    private int state;
    private int array;
    private String worldWideName;
    private boolean nonRedundantAccess;
    private Vector driveAddressCollection;
    private Object enclosureReference;
    private Object arrayReference;

    /** The FibreHardDrive constructor.
      * @param slot - the slot this drive is in
      * @param productInfo
      * @param firmwareLevel 
      * @param state
      * @param pfa
      * @param capacity
      * @param worldWideName - the world wide name of this drive
      * @param nonRedundantAccess
      * @param driveAddressCollection 
      * @param enclosureReference - a reference to the enclosure that owns this drive
      * @param arrayReference - a reference to the array that owns this drive
      */
    public FibreHardDrive(int          slot,
                          ProductInfo  productInfo,
                          String       firmwareLevel,
                          int          state,
                          boolean      pfa,
                          long         capacity,
                          String       worldWideName,
                          boolean      nonRedundantAccess,
                          Vector       driveAddressCollection,
                          Object       enclosureReference,
                          Object       arrayReference)
    {
        super(slot,
              productInfo,
              firmwareLevel,
              pfa,
              capacity);

        this.state = state;
        this.array = array;
        this.worldWideName = worldWideName;
        this.nonRedundantAccess = nonRedundantAccess;
        this.driveAddressCollection = driveAddressCollection;
        this.enclosureReference = enclosureReference;
        this.arrayReference = arrayReference;
    }

    /** Returns this drive's state.
      * @return this drive's state
      */
    public final int getState()
    {
        return state;
    }
    
    /** Returns this drive's world wide name.
      * @return this drive's world wide name
      */
    public final String getWorldWideName()
    {
        return worldWideName;
    }

    /** Returns the drive's redundancy status.
      * @return the drive's redundancy status
      */
    public final boolean getNonRedundantAccess()
    {
        return nonRedundantAccess;
    }
    
    /** Returns this drive's enclosure reference.
      * @return this drive's enclosure reference
      */
    public final Object getEnclosureReference()
    {
        return enclosureReference;
    }

    /** Returns this drive's array reference.
      * @return this drive's array reference
      */
    public final Object getArrayReference()
    {
        return arrayReference;
    }

    /** Returns this drive's driveAddressCollection.
      * @return this drive's driveAddressCollection
      */
    public final Vector getDriveAddressCollection()
    {
        return driveAddressCollection;
    }
    
    /** Changes the parent for this object.
      * @param object  The new parent StorageHotSpareDrives
      * @param addToParent  If true, this object is added to its new parent.
      */
    public void setParent(StorageHotSpareDrives parent, boolean addToParent)
    {
       setParent(parent);

       //Add ourselves to our new parent.
       if (addToParent)
          parent.add(this);
    }

    /** Changes the parent for this object.
      * @param object  The new parent StorageDevices
      * @param addToParent  If true, this object is added to its new parent.
      */
    public void setParent(StorageDevices parent, boolean addToParent)
    {
       setParent(parent);

       //Add ourselves to our new parent.
       if (addToParent)
          parent.add(this);
    }

    /** Returns the data used by the display table model.
      * @return  The multidimensional array of value sets used to populate the console display table.
      */
    public Vector getDisplayDataSet()
    {
        Vector v = super.getDisplayDataSet();

        v.insertElementAt(new Object[] { new ToolTipString("infoHardDriveEnclosure"), String.valueOf(getEnclosure().getId())}, 0);
        v.insertElementAt(new Object[] { new ToolTipString("infoHardDriveRedundancy"), String.valueOf(getRedundancyDisplayStatus())}, 2);
        v.insertElementAt(new Object[] { new ToolTipString("infoHardDriveState"), new StatusString(getDisplayState(), getOverallStatus())}, 3);
        v.insertElementAt(new Object[] { new ToolTipString("infoHardDriveSize"), String.valueOf(getCapacity())}, 5);
        v.addElement(new Object[] { new ToolTipString("infoHardDriveWorldWideName"), String.valueOf(getWorldWideName())});

        int port = 1;
        for (Enumeration e = getDriveAddressCollection().elements(); e.hasMoreElements();)
        {
            v.addElement(new Object[] {new ToolTipString(""), ""});
            FibreInterface fibreInterface = (FibreInterface)e.nextElement();
           
            v.addElement(new Object[] {new ToolTipString("infoDrivePort"), String.valueOf(port++)});
            v.addElement(new Object[] {new ToolTipString("infoDriveChannel"), String.valueOf(fibreInterface.getChannel())});
            v.addElement(new Object[] {new ToolTipString("infoDriveId"), String.valueOf(fibreInterface.getCurrentId())});
        }

        return v;
    } 

    /** Returns the displayable state information.
      * @return the displayable state information
      */
    public String getDisplayState() 
    {
        switch (getState()) 
        {
            case RDY:
                return JCRMUtil.getNLSString("treeStateReady");
            case HSP:
                return JCRMUtil.getNLSString("treeStateHotSpare");
            case DHS:
                return JCRMUtil.getNLSString("treeStateDHS");
            case DDD:
                return JCRMUtil.getNLSString("treeStateDefunct");
            default:
                return JCRMUtil.getNLSString("treeStateOnline");
        }
    }
    
    /** Returns the displayable redundancy information.
      * @return the displayable redundancy information      
      */
    public String getRedundancyDisplayStatus()
    {
        if (getNonRedundantAccess() == false)
            return JCRMUtil.getNLSString("valueDriveRedundancyOk");
        else
            return JCRMUtil.getNLSString("valueDriveRedundancyNotOk");
    }
    
    /** Returns the XML representation of this object.  Can optionally include the standard
      * XML header string.  All children of this object are embedded as XML elements in the output.
      * @param includeHeader  whether to include the XML header
      * @param recurse  whether to embed this object's children
      * @return  the XML representation of this object
      */
    public String toXML(boolean includeHeader, boolean recurse)
    {
        StringBuffer xml = new StringBuffer();

        if (includeHeader)
            xml.append("<?xml version='1.0' encoding='UTF-8'?>").append(SEP);

        xml.append("<FIBREHARDDRIVE type=\"" + String.valueOf(getManufacturer()) + "\">").append(SEP);

        if (recurse)
            for (Enumeration e = enumerateChildren(); e.hasMoreElements();)
                xml.append(((RaidObject) e.nextElement()).toXML(false, true)).append(SEP);
                      
        xml.append("</FIBREHARDDRIVE>").append(SEP);

        return xml.toString();
    }
    
    /** Returns the filename for the icon to be shown in the tree for this RaidObject.
      * @return the filename for the icon to be shown in the tree for this RaidObject
      */
    public String getDisplayIconFilename()
    {
        switch (getState()) 
        {
            case RDY:
                return "s_pdrdy.gif";
            case HSP:
                return "s_pdhsp.gif";
            case DHS:
                return "s_pderr.gif";
            case DDD:
                return "s_pderr.gif";
            default:
                return "s_pdonl.gif";
        }
    }

    /** Returns the string used to identify this object in the tree.
      * @return  the string used to identify this object in the tree
      */
    public String getDisplayName()
    {
        Object[] args = {String.valueOf(getSlot()), getDisplayState(), String.valueOf(getCapacity())};
        return JCRMUtil.makeNLSString("treeHardDrive", args);
    }

    /** Updates the overall status of this object based on its fields. 
      * @param recurse  true if this call should propagate down the config tree
      */
    public void updateOverallStatus(boolean recurse)
    {
       switch (getState())
       {
          // fatal cases
          case DDD:
          case DHS:  setOverallStatus(FATAL); break;

             // normal cases
          case RDY:
          case HSP:
          case ONL: 
          default:   setOverallStatus(NORMAL); break;
       }

       // handle PFA
       if ((getOverallStatus() == NORMAL) && getPfa())
          setOverallStatus(CRITICAL);
    }
    
    /** Creates a deep clone of this object
      * @return a clone of this object
      */
    public Object clone()
    {
       try
       {
          FibreHardDrive clone = (FibreHardDrive) super.clone();
          return clone;
       }
       catch (CloneNotSupportedException e)
       {
          // This should never happen
          throw new InternalError(e.toString());
       }
    }

}


