package com.ibm.sysmgt.raidmgr.dataproc.config.external;

// ---------------------------------------------------------------------
//  Copyright:    (C) International Business Machines Corporation, 2002
//
//  Project Des:  Java Common Raid Manager
//
//  Module Name:  StorageController.java
//
//  Description:
//
//  Revision(s):  Title - Description            Author          Date
//  ------------  ------------------------     -----------    ----------
//  0.0.0         Creation                      DAK            01/30/02

import java.util.*;
import com.ibm.sysmgt.raidmgr.util.*;
import com.ibm.sysmgt.raidmgr.dataproc.util.*;
import com.ibm.sysmgt.raidmgr.dataproc.config.*;

/** The abstract external RAID controller, both hardware and virtual. */
public abstract class StorageController extends RaidObject 
{
    static final long serialVersionUID = 8618997583892197288L;

    /** Mode values */
    public static final boolean PASSIVE = false;
    public static final boolean ACTIVE  = true;

    /** The display headers used by the display table model */
    private final String headers[] =
    {  
        JCRMUtil.getNLSString("infoControllerName"), 
        JCRMUtil.getNLSString("infoControllerValue"),
    };     

    private int              id;
    private CodeVersion      codeVersion;
    private int              status;
    private ProductInfo      productInfo;
    private StorageEnclosure enclosure;

    /** The StorageController constructor.
      * @param id - the id of this controller
      * @param codeVersion - the controllers code version
      * @param status - the controllers status
      * @param productInfo
      */
    public StorageController( int                 id,
                              CodeVersion         codeVersion,  
                              int                 status,
                              ProductInfo         productInfo)
    {
        this.id = id;
        this.codeVersion = codeVersion;  
        this.status = status;
        this.productInfo = productInfo;
    }

    /** Returns this controller's id.
      * @return this controller's id
      */
    public final int getId()
    {
        return id;
    }
    
    /** Returns this controller's CodeVersion.
      * @return this controller's CodeVersion
      */
    public final CodeVersion getCodeVersion()
    {
        return codeVersion;
    }

    /** Returns this controller's ProductInfo.
      * @return this controller's ProductInfo
      */
    public final ProductInfo getProductInfo()
    {
        return productInfo;
    }

    /** Returns this controller's Enclosure.
      * @return this controller's Enclosure
      */
    public final StorageEnclosure getEnclosure()
    {
        return enclosure;
    }
    
    /** Returns the data used by the display table model.
      * @return  The multidimensional array of value sets used to populate the console display table.
      */
    public Vector getDisplayDataSet()
    {
        Vector v = new Vector();
        if (getStatus() != REMOVED)
        {
            if ((getEnclosure().getId()) != (StorageEnclosure.CONTROLLER_ENCLOSURE))
                v.addElement(new Object[] { new ToolTipString("infoEnclosureNumber"), String.valueOf(getEnclosure().getId())});

            v.addElement(new Object[] { new ToolTipString("infoControllerProductId"), getProductInfo().getProductId()});
            v.addElement(new Object[] { new ToolTipString("infoControllerManufacturer"), getProductInfo().getManufacturer()});
            v.addElement(new Object[] { new ToolTipString("infoControllerSerialNumber"), getProductInfo().getSerialNumber()});
        }
        return v;
    }

    /** Returns the data headers used by the display table model.
      * @return  The array of value set names used to populate the console display table.
      */
    public String[] getDisplayDataSetNames()
    {
        return headers;
    }

    /** Returns the string used to identify this object in the window title bar.
      * @return  the string used to identify this object in the window title bar
      */
    public String getWindowDisplayName()
    {
        Object[] args = { getDisplayID()};
        return(JCRMUtil.getNLSString("titleGUI") + " " + JCRMUtil.makeNLSString("titleControllerInfo", args));
    }

    /** Returns the current adapter status.  Values are defined in this class.
      * @return  the adapter status
      */
    public final int getStatus()
    {
        return status;
    }

    /** Changes the parent for this object.
      * @param object  The new parent StorageEnclosure
      * @param addToParent  If true, this object is added to its new parent.
      */
    public void setParent(StorageEnclosure parent, boolean addToParent)
    {
        setParent(parent);
        enclosure = parent;

        if (addToParent)
            parent.add(this);
    }

    /** Determine equivalence.
      * @param object  The object to compare to this one
      * @return  True if the objects are functionally equivalent, false otherwise.
      */
    public boolean equals(Object object)
    {
        // Do this first, since it's fast.
        if (this == object)
            return true;

        // We DON'T subclass java.lang.Object, so we must check for superclass equality.
        if (!super.equals(object))
            return false;

        // Finally, we can do the cast and compare relevant fields.
        StorageController controller = (StorageController) object;

        // Compare all relevant fields
        if (getId() != controller.getId())
            return false;

        // Everything matched, so we're equal!
        return true;
    }

    /** Satisfy comparable interface.  This method controls ordering in the tree.
      * Ordering of PhysicalDevice object is based on deviceID.
      * @param object  The object to compare to this one
      * @return  0 for equality, <0 if this object comes first, >0 if the passed-in object comes first
      */ 
    public int compareTo(Object object) throws ClassCastException
    {
        if (object instanceof StorageDevice)
            return -1;

        if (!(object instanceof StorageController))
            throw new ClassCastException("Object of type " + object.getClass().getName() + 
                                         " may not be compared to " + this.getClass().getName());
        else
        {
            StorageController controller = (StorageController) object;
            return(getId() - controller.getId());
        }
    }

    /** Since we override equals(), we must override hashcode().
      * @return a hash value for this object
      */
    public int hashCode()
    {
        int result = 0;
        result ^= super.hashCode();
        result = PRIME*result + getId();
        return result;
    }
}

