package com.ibm.sysmgt.raidmgr.dataproc.config.external;

// ---------------------------------------------------------------------
//  Copyright:    (C) International Business Machines Corporation, 2002
//
//  Project Des:  Java Common Raid Manager
//
//  Module Name:  StorageDevices.java
//
//  Description:
//
//  Revision(s):  Title - Description            Author              Date
//  ------------  ------------------------     ---------------    ----------
//  0.0.0         Creation                      Dean Kalman        02/14/02

import java.util.*;
import java.io.*;
import com.ibm.sysmgt.raidmgr.util.*;
import com.ibm.sysmgt.raidmgr.common.*;
import com.ibm.sysmgt.raidmgr.dataproc.util.*;
import com.ibm.sysmgt.raidmgr.dataproc.config.*;

/** This class is the container object for physical devices.  
  * The only allowable children of PhysicalDevices are PhysicalDevice, as enforced 
  * by the add() method in this class.  
  */
public class StorageDevices extends RaidObject 
{
    static final long serialVersionUID = 4203833482229750187L;

    /** The display headers used by the display table model */
    private final String headers[] =
    { 
        JCRMUtil.getNLSString("infoPhysDrivesName"), 
        JCRMUtil.getNLSString("infoPhysDrivesValue"),
    };

    /** The StorageDevices constructor.
      */
    public StorageDevices()
    {
    }

    /** Changes the parent for this object.
      * @param object  The new parent StorageArray
      * @param addToParent  If true, this object is added to its new parent.
      */
    public void setParent(StorageGroup parent, boolean addToParent)
    {
        setParent(parent);

        //Add ourselves to our new parent.
        if (addToParent)
            parent.add(this);
    }

    /** Returns an enumeration of the devices in this container.
      * @return an enumeration of the devices in this container
      */
    public Enumeration enumerateDevices()
    {
        Vector devices = new Vector();

        for (Enumeration e = enumerateChildren(); e.hasMoreElements();)
        {
            Object object = e.nextElement();
            if (object instanceof StorageDevice)
                devices.addElement(object);
        }
        return devices.elements();
    }

    /** This method returns a Vector of the physical device objects.
      * @param filter  If null, all physical devices are returned.  If non-null, return physical devices passed by the filter.
      * @return the Vector of physical device objects
      */
    public Vector getDeviceCollection(StorageDeviceFilterIntf filter)
    {
        Vector devices = new Vector();
        for (Enumeration e = enumerateDevices(); e.hasMoreElements();)
        {
            StorageDevice device = (StorageDevice) e.nextElement();
            if ((filter == null) || (filter.accept(device)))
                devices.addElement(device);
        }
        return devices;
    }

    /** Returns the data used by the display table model.
      * @return  The multidimensional array of value sets used to populate the console display table.
      */
    public Vector getDisplayDataSet()
    {
       Vector v = new Vector();

       v.addElement(new Object[] {new ToolTipString("infoStorageArrayDriveCount"), String.valueOf(getDeviceCollection(null).size())});

       return v;
    }

    /** Returns the data headers used by the display table model.
      * @return  The array of value set names used to populate the console display table.
      */
    public String[] getDisplayDataSetNames()
    {
        return headers;
    }

    /** Returns the string used to identify this object in the tree.
      * @return  the string used to identify this object in the tree
      */
    public String getDisplayName()
    {
       RaidObject parent = getParent();

       if (parent instanceof StorageGroup)
          return JCRMUtil.getNLSString("treePhysDrivesInArray") + ' ' + parent.getDisplayID();
       else
          return JCRMUtil.getNLSString("treePhysicalDrives");
    }
    
    /** Returns the string used to identify this object in the window title bar.
      * @return  the string used to identify this object in the window title bar
      */
    public String getWindowDisplayName()
    {
        return(JCRMUtil.getNLSString("titleGUI"));
    }

    /** Returns the filename for the icon to be shown in the tree for this RaidObject.
      * @returns the filename for the icon to be shown in the tree for this RaidObject
      */
    public String getDisplayIconFilename()
    {
        switch (getOverallStatus())
        {
        case NORMAL:    return "s_phdrs1.gif";
        case CRITICAL:  return "s_phdrs2.gif";
        case FATAL:     return "s_phdrs3.gif";
        default:        return "s_phdrs1.gif";
        }
    }

    /** Satisfy comparable interface.  This method controls ordering in the tree.
      * @param object  The object to compare to this one
      * @return  0 for equality, -1 if this object comes first, 1 if the passed-in object comes first
      */ 
    public int compareTo(Object object) throws ClassCastException
    {
        if (object instanceof StorageLogicalDrives)
            return 1;
        else
            throw new ClassCastException("Object of type " + object.getClass().getName() + 
                                         " may not be compared to " + this.getClass().getName());
    }

    /** Returns the XML representation of this object.  Can optionally include the standard
      * XML header string.  All children of this object are embedded as XML elements in the output.
      * @param includeHeader  whether to include the XML header
      * @param recurse  whether to embed this object's children
      * @return  the XML representation of this object
      */
    public String toXML(boolean includeHeader, boolean recurse)
    {
        StringBuffer xml = new StringBuffer();

        if (includeHeader)
            xml.append("<?xml version='1.0' encoding='UTF-8'?>").append(SEP);

        xml.append("<STORAGEDEVICES>").append(SEP);

        if (recurse)
            for (Enumeration e = enumerateChildren(); e.hasMoreElements();)
                xml.append(((RaidObject) e.nextElement()).toXML(false, true)).append(SEP);

        xml.append("</STORAGEDEVICES>").append(SEP);

        return xml.toString();
    }

    /** Add a StorageDevice object to this object.
      * @param object  a StorageDevice
      */
    public void add(StorageDevice device)
    {
        super.add(device);
    }

    /** Determine equivalence.
      * @param object  The object to compare to this one
      * @return  True if the objects are functionally equivalent, false otherwise.
      */
    public boolean equals(Object object)
    {
        // Do this first, since it's fast.
        if (this == object)
            return true;

        // We DON'T subclass java.lang.Object, so we must check for superclass equality.
        if (!super.equals(object))
            return false;

        // Finally, we can do the cast and compare relevant fields.
        StorageDevices devices = (StorageDevices) object;

        // not equal if either parent reference is null, but not both
        if ((getParent() == null) != (devices.getParent() == null))
            return false;

        // not equal if parent is not null but equals fails    
        if ((getParent() != null) && !getParent().equals(devices.getParent()))
            return false;

        if (!getParent().equals(devices.getParent()))
            return false;

        // Everything matched, so we're equal!
        return true;
    }

    /** Since we override equals(), we must override hashcode().
      * @return a hash value for this object
      */
    public int hashCode()
    {
        int result = 0;
        result ^= super.hashCode();
        result = PRIME*result + ((getParent() == null) ? 0 : getParent().hashCode());
        return result;
    }
}
