package com.ibm.sysmgt.raidmgr.dataproc.config.external;

// ---------------------------------------------------------------------
//  Copyright:    (C) International Business Machines Corporation, 2002
//
//  Project Des:  Java Common Raid Manager
//
//  Module Name:  StorageEnclosure.java
//
//  Description:
//
//  Revision(s):  Title - Description            Author          Date
//  ------------  ------------------------     -----------    ----------
//  0.0.0         Creation                      DAK            01/30/02

import java.util.*;
import com.ibm.sysmgt.raidmgr.util.*;
import com.ibm.sysmgt.raidmgr.dataproc.util.*;
import com.ibm.sysmgt.raidmgr.dataproc.config.*;

/** The abstract external storage enclosure. A storage enclosure can hold 1 or more controllers and or
  * 1 or more hard drives 
  */
public abstract class StorageEnclosure extends RaidObject 
{
    static final long serialVersionUID = -148946687830622415L;
    public static final int CONTROLLER_ENCLOSURE = -1;

    /** The display headers used by the display table model */
    private final String headers[] =
    {  
        JCRMUtil.getNLSString("infoControllerEnclosureName"), 
        JCRMUtil.getNLSString("infoControllerEnclosureValue"),
    };       

    private int               id;
    private int               type;
    private ProductInfo       productInfo;
    private StorageComponents storageComponents;
    private StorageSubsystem  storageSubsystem;
    private RaidSystem  system;


    /** The StorageEnclosure constructor.
      * @param id - the id of this enclosure
      * @param int - the type of this enclosure
      * @param productInfo - the product info for this enclosure
      * @param StorageComponents 
      */
    public StorageEnclosure(int               id,
                            int               type,
                            ProductInfo       productInfo,
                            StorageComponents storageComponents)  
    {
        this.id = id;
        this.type = type;
        this.productInfo = productInfo;     
        this.storageComponents = storageComponents;
    }

    /** Returns this enclosure's id.
      * @return this enclosure's id
      */
    public final int getId()
    {
        return id;
    }

    /** Returns this enclosure's type.
      * @return this enclosure's type
      */
    public final int getType()
    {
        return type;
    }
    
    /** Returns this enclosure's ProductInfo.
      * @return this enclosure's ProductInfo
      */
    public final ProductInfo getProductInfo()
    {
        return productInfo;
    }

    /** Returns this enclosure's storage components.
      * @return this enclosure's storage components
      */
    public final StorageComponents getStorageComponents()
    {
        return storageComponents;
    }

    /** Returns an enumeration of the devices in this container.
      * @return an enumeration of the devices in this container
      */
    public Enumeration enumerateDevices()
    {
        Vector devices = new Vector();

        for (Enumeration e = enumerateChildren(); e.hasMoreElements();)
        {
            Object object = e.nextElement();
            if (object instanceof StorageDevice)
                devices.addElement(object);
        }
        return devices.elements();
    }

    /** This method returns a Vector of the physical device objects.
      * @param filter  If null, all physical devices are returned.  If non-null, return physical devices passed by the filter.
      * @return the Vector of physical device objects
      */
    public Vector getDeviceCollection(StorageDeviceFilterIntf filter)
    {
        Vector devices = new Vector();
        for (Enumeration e = enumerateDevices(); e.hasMoreElements();)
        {
            StorageDevice device = (StorageDevice) e.nextElement();
            if ((filter == null) || (filter.accept(device)))
                devices.addElement(device);
        }
        return devices;
    }

    /** Returns an enumeration of the controllers in this container.
      * @return an enumeration of the controllers in this container
      */
    public Enumeration enumerateControllers()
    {
        Vector controllers = new Vector();

        for (Enumeration e = enumerateChildren(); e.hasMoreElements();)
        {
            Object object = e.nextElement();
            if (object instanceof StorageController)
                controllers.addElement(object);
        }
        return controllers.elements();
    }

    /** This method returns a Vector of the controller device objects.
      * @param filter  If null, all physical devices are returned.  If non-null, return physical devices passed by the filter.
      * @return the Vector of physical device objects
      */
    public Vector getControllerCollection(StorageControllerFilterIntf filter)
    {
        Vector controllers = new Vector();
        for (Enumeration e = enumerateControllers(); e.hasMoreElements();)
        {
            StorageController controller = (StorageController) e.nextElement();
            if ((filter == null) || (filter.accept(controller)))
                controllers.addElement(controller);
        }
        return controllers;
    }

    /** This method returns the number of controllers in this storage enclosure
      * @return the number of controllers in this storage enclosure
      */
    public int getControllerCount()
    {
       Vector controllers = new Vector();

       for (Enumeration e = enumerateChildren(); e.hasMoreElements();)
       {
           Object object = e.nextElement();
           if (object instanceof StorageController)
               controllers.addElement(object);
       }
       return controllers.size();
    }

    /** This method returns a Vector of the array objects contained in this object.
      * @param filter  If null, all arrays are returned.  If non-null, return arrays passed by the filter.
      * @return the Vector of array objects
      */
    public Vector getArrayCollection(StorageGroupFilterIntf filter)
    {
        return storageSubsystem.getStorageArraysContainer().getArrayCollection(filter); 
    }

    /** This method returns an enumeration of the logical drives owned by this storage enclosure.
      * @return an enumeration of the logical drives
      */
    public Enumeration enumerateLogicalDrives()
    {
      Vector v = new Vector();
      for (Enumeration e = getArrayCollection(null).elements(); e.hasMoreElements();)
      {
         Array array = (Array)e.nextElement();
         v.addAll(array.getLogicalDriveCollection(null)); 
      }
      return v.elements();
    }
    
    /** Updates the overall status for this enclosure. 
      * @param recurse  true if this call should propagate down the config tree
      */
    public void updateOverallStatus(boolean recurse)
    {
        // Recursively update the status of all children
        if (recurse)
        {
           for (Enumeration e = enumerateChildren(); e.hasMoreElements();)
              ((RaidObject) e.nextElement()).updateOverallStatus(recurse);
        }

        // No children
        if (getChildCount() == 0)
           setOverallStatus(NORMAL);

        // Check for all FATAL children condition
        else if (getChildrenByStatus(FATAL).size() == getChildCount())
           setOverallStatus(FATAL);

        // Check for all NORMAL children condition
        else if (getChildrenByStatus(NORMAL).size() == getChildCount())
           setOverallStatus(NORMAL);

        // In-between is FATAL
        else
           setOverallStatus(FATAL);


        // Override the children status if the enclosure is status fatal
        for (Enumeration e = getStorageComponents().getComponents(); e.hasMoreElements();)
        {
            StorageComponent component = (StorageComponent)e.nextElement();

            if (component.getOverallStatus() == FATAL)
            {
                setOverallStatus(FATAL);
                break;
            }
        }
    }
    
    /** Returns the data headers used by the display table model.
      * @return  The array of value set names used to populate the console display table.
      */
    public String[] getDisplayDataSetNames()
    {
        return headers;
    }

    /** Returns the string used to identify this object in the window title bar.
      * @return  the string used to identify this object in the window title bar
      */
    public String getWindowDisplayName()
    {
        Object[] args = { getDisplayID()};
        return(JCRMUtil.getNLSString("titleGUI") + " " + JCRMUtil.makeNLSString("titleControllerEnclosureInfo", args));
    }

    /** Changes the parent for this object.
      * @param object  The new parent RaidSystem
      * @param addToParent  If true, this object is added to its new parent.
      */
    public void setParent(StorageSubsystem parent, boolean addToParent)
    {
        setParent(parent);
        storageSubsystem = parent;

        if (addToParent)
            parent.add(this);
    } 

    /** Changes the parent for this object.
      * @param object  The new parent RaidSystem
      * @param addToParent  If true, this object is added to its new parent.
      */
    public void setParent(RaidSystem parent, boolean addToParent)
    {
        setParent(parent);
        system = parent;

        if (addToParent)
            parent.add(this);
    }


    /** A StorageEnclosure can contain StorageController objects.
      * @param object a StorageController
      */
    public void add(StorageController object)
    {
       super.add(object);
    }

    /** A StorageEnclosure can contain storage device objects.
      * @param object a StorageDevice
      */
    public void add(StorageDevice object)
    {
       super.add(object);
    }

    /** Returns this storage enclosure's RaidSystem.
      * @return this storage enclosure's RaidSystem
      */
    public final RaidSystem getRaidSystem()
    {
       return system;
    }
    
    /** Returns the filename for the icon to be shown in the tree for this RaidObject.
      * @returns the filename for the icon to be shown in the tree for this RaidObject
      */
    public String getDisplayIconFilename()
    {
       if (getLookAndFeel() == IBIS)
          return "ibis/icon-enclosure.gif";
       else
       {
         switch (getOverallStatus())
         {
            case NORMAL:    return "s_proc.gif";
            case CRITICAL:  return "s_proc2.gif";
            case FATAL:     return "s_proc3.gif";
            default:        return "s_proc.gif";
         }
       }
    }

    /** Determine equivalence.
      * @param object  The object to compare to this one
      * @return  True if the objects are functionally equivalent, false otherwise.
      */
    public boolean equals(Object object)
    {
        // Do this first, since it's fast.
        if (this == object)
            return true;

        // We DON'T subclass java.lang.Object, so we must check for superclass equality.
        if (!super.equals(object))
            return false;

        // Finally, we can do the cast and compare relevant fields.
        StorageEnclosure enclosure = (StorageEnclosure) object;

        // Compare all relevant fields
        //  if (getEnclosure() != controller.getEnclosure())
        //     return false;

        if (getId() != enclosure.getId())
            return false;

        // Everything matched, so we're equal!
        return true;
    }

    /** Satisfy comparable interface.  This method controls ordering in the tree.
      * Ordering of PhysicalDevice object is based on deviceID.
      * @param object  The object to compare to this one
      * @return  0 for equality, <0 if this object comes first, >0 if the passed-in object comes first
      */ 
    public int compareTo(Object object) throws ClassCastException
    {

        if (object instanceof StorageEnclosure)
        {
            StorageEnclosure enclosure = (StorageEnclosure) object;
            return(getId() - enclosure.getId());
        }
        else if (object instanceof StorageArrays)
            return -1;
        else if (object instanceof StorageLogicalDrives)
            return -1;
        else if (object instanceof StorageHotSpareDrives)
            return -1;
        else
            throw new ClassCastException("Object of type " + object.getClass().getName() + 
                                         " may not be compared to " + this.getClass().getName());
    }
    
    /** Since we override equals(), we must override hashcode().
      * @return a hash value for this object
      */
    public int hashCode()
    {
        int result = 0;
        result ^= super.hashCode();
        result = PRIME*result + getId();
        return result;
    }

    /** Sets this enclosure's id.
      * @param set this enclosure's id
      */
    public void setId(int id)
    {
        this.id = id;
    }


}

