package com.ibm.sysmgt.raidmgr.dataproc.config.external;

// ---------------------------------------------------------------------
//  Copyright:    (C) International Business Machines Corporation, 1998
//
//  Project Des:  Java Common Raid Manager
//
//  Module Name:  StorageGroup.java
//
//  Description:
//
//  Revision(s):  Title - Description            Author          Date
//  ------------  ------------------------     -----------    ----------
//  0.0.0         Creation                      Dean Kalman    02/12/02

import java.util.*;
import com.ibm.sysmgt.raidmgr.util.*;
import com.ibm.sysmgt.raidmgr.dataproc.util.*;
import com.ibm.sysmgt.raidmgr.dataproc.config.*;

public abstract class StorageGroup extends RaidObject 
{
   static final long serialVersionUID = 8082673963671194841L;

   /** The display headers used by the display table model */
   private final String headers[] =
   {  
      JCRMUtil.getNLSString("infoArrayName"), 
      JCRMUtil.getNLSString("infoArrayValue")
   };

   private int                  id;
   private long                 freeSpace;
   private StorageLogicalDrives logicalDrives;
   private StorageDevices       devices;

   /** The StorageArray constructor.
     * @param id - the id of this array
     * @param freeSpace - the amount of free space available in this array
     */
   public StorageGroup(int id,
                       long freeSpace)
   {
      this.id        = id;
      this.freeSpace = freeSpace;
      
      logicalDrives = new StorageLogicalDrives();
      logicalDrives.setParent(this, false);
      super.add(logicalDrives);

      devices = new StorageDevices();
      devices.setParent(this, false);
      super.add(devices);
   }

   /** Returns the array Id
     * @return the array Id
     */
   public final int getId()
   {
      return id;
   }

   /** Returns the array free space
     * @return the array free space
     */
   public final long getFreeSpace()
   {
      return freeSpace;
   }

   /** Returns the size of this array
     * @return the size of this array
     */
   public final long getSize()
   {
      long size = 0;
      for (Enumeration e = getStorageDevicesContainer().enumerateChildren(); e.hasMoreElements();)
      {
          StorageDevice device = (StorageDevice)e.nextElement();

          size += device.getCapacity();
      }
      return size;
   }

   /** Returns the storage logical drives container
     * @return the storage logical drives container
     */
   public StorageLogicalDrives getStorageLogicalDrivesContainer() 
   {
       return logicalDrives;
   }

   /** Returns the storage devices container
     * @return the storage devices container
     */
   public StorageDevices getStorageDevicesContainer() 
   {
       return devices;
   }

   /** Returns the displayable Id
     * @return the displayable Id
     */
   public final String getDisplayID()
   {
      return String.valueOf(id);
   }

   
   /** Returns the data used by the display table model.
     * @return  The multidimensional array of value sets used to populate the console display table.
     */
   public Vector getDisplayDataSet()
   {
       Vector v = new Vector();

       v.addElement(new Object[] { new ToolTipString("infoStorageArrayIdentifier"), String.valueOf(getId())});

       return v;
   }

   /** Returns the data headers used by the display table model.
     * @return  The array of value set names used to populate the console display table.
     */
   public String[] getDisplayDataSetNames()
   {
      return new String[]
      {  
         JCRMUtil.getNLSString("infoArrayName"), 
         JCRMUtil.getNLSString("infoArrayValue")
      };
   }

   /** Returns the string used to identify this object in the window title bar.
     * @return  the string used to identify this object in the window title bar
     */
   public String getWindowDisplayName()
   {
      Object[] args = { String.valueOf(getId()) };
      return(JCRMUtil.getNLSString("titleGUI") + " " + JCRMUtil.makeNLSString("titleSubsystemInfo", args));
   }

   /** Changes the parent for this object.
     * @param object  The new parent RaidSystem
     * @param addToParent  If true, this object is added to its new parent.
     */
   public void setParent(StorageArrays parent, boolean addToParent)
   {
       setParent(parent);

       if (addToParent)
           parent.add(this);
   }

   /** A StorageGroup can contain StorageLogicalDrives objects.
     * @param object  a StorageLogicalDrive
     */
   public void add(StorageLogicalDrives object)
   {
       super.add(object);
   }

   /** A StorageGroup can contain StorageDevices objects.
     * @param object  a StorageDevices object
     */
   public void add(StorageDevices object)
   {
       super.add(object);
   }

   /** Returns an enumeration of the StorageLogicalDrives in this container.
     * @return an enumeration of the StorageLogicalDrives in this container
     */
   public Enumeration enumerateLogicalDrives()
   {
       StorageLogicalDrives storageLogicalDrives = getStorageLogicalDrivesContainer();
       return storageLogicalDrives.getLogicalDriveCollection(null).elements();
   }


   /** This method returns a Vector of the StorageLogicalDrive objects that belong to this storage group.
     * @param filter  If null, all StorageLogicalDrivess are returned.  If non-null, return StorageLogicalDrives
     *                passed by the filter.
     * @return the Vector of StorageLogicalDrive objects
     */
   public Vector getLogicalDriveCollection(StorageLogicalDriveFilterIntf filter)
   {
       Vector logicalDrives = new Vector();
       for (Enumeration e = enumerateLogicalDrives(); e.hasMoreElements();)
       {
           StorageLogicalDrive logicalDrive = (StorageLogicalDrive) e.nextElement();
           if ((filter == null) || (filter.accept(logicalDrive)))
               logicalDrives.addElement(logicalDrive);
       }
       return logicalDrives;
   }
   
   /** Returns an enumeration of the devices in this container.
     * @return an enumeration of the devices in this container
     */
   public Enumeration enumerateDevices()
   {
       StorageDevices devices = getStorageDevicesContainer();
       return devices.getDeviceCollection(null).elements();
   }


   /** This method returns a Vector of the StorageDevice objects that belong to this storage group.
     * @param filter  If null, all StorageDevice are returned.  If non-null, return StorageDevices
     *                passed by the filter.
     * @return the Vector of StorageDevice objects
     */
   public Vector getDeviceCollection(StorageDeviceFilterIntf filter)
   {
       Vector devices = new Vector();
       for (Enumeration e = enumerateDevices(); e.hasMoreElements();)
       {
           StorageDevice device = (StorageDevice) e.nextElement();
           if ((filter == null) || (filter.accept(device)))
               devices.addElement(device);
       }
       return devices;
   }
   
   /** Returns the filename for the icon to be shown in the tree for this RaidObject.
     * @returns the filename for the icon to be shown in the tree for this RaidObject
     */
   public String getDisplayIconFilename()
   {
      switch (getOverallStatus())
      {
         default:
         case NORMAL:    return "s_array1.gif";
         case CRITICAL:  return "s_array2.gif";
         case FATAL:     return "s_array3.gif";
      }
   }

   /** Updates the overall status for this storage group.  This method's behavior
     * is similar to that in RaidObject, except that it only considers
     * its children that are storage logical drive objects.  For this reason,
     * the overall status of a storage group will always mirror that of its
     * StorageLogicalDrives container.
     * @param recurse  true if this call should propagate down the config tree
     */
   public void updateOverallStatus(boolean recurse)
   {
      // Recursively update the status of all children
      if (recurse)
      {
         for (Enumeration e = enumerateChildren(); e.hasMoreElements();)
            ((RaidObject) e.nextElement()).updateOverallStatus(recurse);
      }
      // these containers aren't children in the case of sub-arrays, so do them explicitly
      logicalDrives.updateOverallStatus(recurse);
      devices.updateOverallStatus(recurse);

      setOverallStatus(logicalDrives.getOverallStatus());
   }
   
   /** Determine equivalence.
     * @param object  The object to compare to this one
     * @return  True if the objects are functionally equivalent, false otherwise.
     */
   public boolean equals(Object object)
   {
      // Do this first, since it's fast.
      if (this == object)
         return true;

      // We DON'T subclass java.lang.Object, so we must check for superclass equality.
      if (!super.equals(object))
         return false;

      // Finally, we can do the cast and compare relevant fields.
      StorageGroup array = (StorageGroup) object;
      
      if (id != array.id)
         return false;

      // Everything matched, so we're equal!
      return true;
   }


   /** Satisfy comparable interface.  This method controls ordering in the tree.
     * Ordering of PhysicalDevice object is based on deviceID.
     * @param object  The object to compare to this one
     * @return  0 for equality, <0 if this object comes first, >0 if the passed-in object comes first
     */ 
   public int compareTo(Object object) throws ClassCastException
   {
       if (!(object instanceof StorageGroup))
           throw new ClassCastException("Object of type " + object.getClass().getName() + 
                                        " may not be compared to " + this.getClass().getName());
       else
       {
           StorageGroup array = (StorageGroup) object;
           return(getId() - array.getId());
       }
   }

   /** Since we override equals(), we must override hashcode().
     * @return a hash value for this object
     */
   public int hashCode()
   {
      int result = 0;
      result ^= super.hashCode();
      result = PRIME*result + id;
      return result;
   }
}
