package com.ibm.sysmgt.raidmgr.dataproc.config.external;

// ---------------------------------------------------------------------
//  Copyright:    (C) International Business Machines Corporation, 2002
//
//  Project Des:  Java Common Raid Manager
//
//  Module Name:  StorageHotSpareDrives.java
//
//  Description:
//
//  Revision(s):  Title - Description            Author              Date
//  ------------  ------------------------     ---------------    ----------
//  0.0.0         Creation                      Dean Kalman       02/11/02

import java.util.*;
import com.ibm.sysmgt.raidmgr.util.*;
import com.ibm.sysmgt.raidmgr.dataproc.util.*;
import com.ibm.sysmgt.raidmgr.dataproc.config.*;

/** This class is the container object at the Adapter level for hot spare HardDrive drive objects.  
  * The only allowable children of HotSpareDrives are HardDrive objects, as enforced 
  * by the add() method in this class.  As a well-behaved RaidObject, this class uses
  * private final fields to enforce identity integrity.
  */
public class StorageHotSpareDrives extends RaidObject 
{
   static final long serialVersionUID = -8382471130333816399L;

   /** The display headers used by the display table model */
   private final String headers[] =
   { 
      JCRMUtil.getNLSString("infoHotSparesName"), 
      JCRMUtil.getNLSString("infoHotSparesValue"),
   };

   /** The HotSpareDrives constructor.
     */
   public StorageHotSpareDrives()
   {
   }
   
   /** Returns an enumeration of the arrays in this container.
     * @return an enumeration of the arrays in this container
     */
   public Enumeration enumerateDevices()
   {
       Vector devices = new Vector();

       for (Enumeration e = enumerateChildren(); e.hasMoreElements();)
       {
           Object object = e.nextElement();
           if (object instanceof StorageDevice)
               devices.addElement(object);
       }
       return devices.elements();
   }

   /** This method partially satisfies StorageDeviceCollectionIntf.  It returns a Vector of the
     * StorageDevice objects in this container.
     * @param filter  If null, all objects are returned.  If non-null, return objects passed by the filter.
     * @return the Vector of StorageDevice objects
     */
   public Vector getPhysicalDeviceCollection(StorageDeviceFilterIntf filter)
   {
      Vector devices = new Vector();
      for (Enumeration e = enumerateDevices(); e.hasMoreElements();)
      {
         Object object = e.nextElement();
         if (object instanceof StorageDevice)
         {
            StorageDevice device = (StorageDevice) object;
            if ((filter == null) || (filter.accept(device)))
               devices.addElement(device);
         }
      }
      return devices;
   }
   
   /** Changes the parent for this object.
     * @param object  The new parent Adapter
     * @param addToParent  If true, this object is added to its new parent.
     */
   public void setParent(StorageSubsystem parent, boolean addToParent)
   {
      setParent(parent);

      //Add ourselves to our new parent.
      if (addToParent)
         parent.add(this);
   }

   /** Returns the data used by the display table model.
     * @return  The multidimensional array of value sets used to populate the console display table.
     */
   public Vector getDisplayDataSet()
   {
      Vector v = new Vector();

      v.addElement(new Object[]
                   {
                      new ToolTipString("infoNumberOfHotSpares"), 
                      String.valueOf(getChildCount())
                   });

      return v;
   }

   /** Returns the data headers used by the display table model.
     * @return  The array of value set names used to populate the console display table.
     */
   public String[] getDisplayDataSetNames()
   {
      return new String[]
      { 
         JCRMUtil.getNLSString("infoHotSparesName"), 
         JCRMUtil.getNLSString("infoHotSparesValue"),
      };
   }

   /** Returns the string used to identify this object in the tree.
     * @return  the string used to identify this object in the tree
     */
   public String getDisplayName()
   {
      return JCRMUtil.getNLSString("treeHotSpares");
   }

   /** Returns the string used to identify this object in the window title bar.
     * @return  the string used to identify this object in the window title bar
     */
   public String getWindowDisplayName()
   {
      return(JCRMUtil.getNLSString("titleGUI"));
   }

   /** Returns the XML representation of this object.  Can optionally include the standard
     * XML header string.  All children of this object are embedded as XML elements in the output.
     * @param includeHeader  whether to include the XML header
     * @param recurse  whether to embed this object's children
     * @return  the XML representation of this object
     */
   public String toXML(boolean includeHeader, boolean recurse)
   {
      StringBuffer xml = new StringBuffer();

      if (includeHeader)
         xml.append("<?xml version='1.0' encoding='UTF-8'?>").append(SEP);

      xml.append("<STORAGEHOTSPAREDRIVES>").append(SEP);

      if (recurse)
         for (Enumeration e = enumerateChildren(); e.hasMoreElements();)
            xml.append(((RaidObject) e.nextElement()).toXML(false, true)).append(SEP);

      xml.append("</STORAGEHOTSPAREDRIVES>").append(SEP);

      return xml.toString();
   }

   /** An HotSpareDrives container can contain only FibreHardDrive objects.
     * @param object  a FibreHardDrive
     */
   public void add(StorageDevice object)
   {
      super.add(object);
   }

   /** Satisfy comparable interface.  This method controls ordering in the tree.
     * The tree peers of Arrays are LogicalDrives, PhysicalDrives, Enclosures,
     * and HotSpareDrives, only one of which will exist per RaidSystem.  
     * Their tree order is:
     *
     * RaidSystem
     *    Arrays
     *    LogicalDrives
     *    HotSpareDrives
     *    PhysicalDrives
     *    Enclosures
     *
     * @param object  The object to compare to this one
     * @return  0 for equality, -1 if this object comes first, 1 if the passed-in object comes first
     */ 
   public int compareTo(Object object) throws ClassCastException
   {
      if (object instanceof StorageSubsystem)
         return 1;
      if (object instanceof StorageArrays)
         return 1;
      else if (object instanceof StorageLogicalDrives)
         return 1;
      else if (object instanceof StorageHotSpareDrives)
         return 0;
      else
         throw new ClassCastException("Object of type " + object.getClass().getName() + 
                                      " may not be compared to " + this.getClass().getName());
   }

   /** Returns the filename for the icon to be shown in the tree for this RaidObject.
     * @returns the filename for the icon to be shown in the tree for this RaidObject
     */
   public String getDisplayIconFilename()
   {
      switch (getOverallStatus())
      {
         case NORMAL:    return "s_hsps1.gif";
         case CRITICAL:  return "s_hsps2.gif";
         case FATAL:     return "s_hsps3.gif";
         default:        return "s_hsps1.gif";
      }
   }

   /** Determine equivalence.
     * @param object  The object to compare to this one
     * @return  True if the objects are functionally equivalent, false otherwise.
     */
   public boolean equals(Object object)
   {
      // Do this first, since it's fast.
      if (this == object)
         return true;

      // We DON'T subclass java.lang.Object, so we must check for superclass equality.
      if (!super.equals(object))
         return false;

      // Finally, we can do the cast and compare relevant fields.
      StorageHotSpareDrives drives = (StorageHotSpareDrives) object;

      // Everything matched, so we're equal!
      return true;
   }

   /** Removes a child RaidObject from this RaidObject.
     * @param object  the RaidObject to remove
     * @return  true if the specified RaidObject was a child of this RaidObject, false otherwise.
     */
   public boolean remove(RaidObject object)
   {
      return super.remove(object);
   }

   /** Since we override equals(), we must override hashcode().
     * @return a hash value for this object
     */
   public int hashCode()
   {
      int result = 0;
      result ^= super.hashCode();
      return result;
   }
}
