package com.ibm.sysmgt.raidmgr.dataproc.config.external;

// ---------------------------------------------------------------------
//  Copyright:    (C) International Business Machines Corporation, 2002
//
//  Project Des:  Java Common Raid Manager
//
//  Module Name:  StorageLogicalDrive.java
//
//  Description:
//
//  Revision(s):  Title - Description            Author          Date
//  ------------  ------------------------     -----------    ----------
//  0.0.0         Creation                     Dean Kalman     02/12/02

import java.util.*;
import com.ibm.sysmgt.raidmgr.util.*;
import com.ibm.sysmgt.raidmgr.dataproc.util.*;
import com.ibm.sysmgt.raidmgr.dataproc.config.*;

/** Represents a Storage Logical Drive. */
public abstract class StorageLogicalDrive extends RaidObject 
{
    static final long serialVersionUID = 3230553982756874092L;

    /** The display headers used by the display table model */
    private final String headers[] =
    {  
        JCRMUtil.getNLSString("infoLogDriveName"), 
        JCRMUtil.getNLSString("infoLogDriveValue"),
    };

    private int     id;
    private int     state;
    private long    size;
    private int     raidLevel;
    private int     array;
    private int     controller;

    /** Creates a StorageLogicalDrive object with the given parameters.
      * @param id            - the id of this logical drive
      * @param state         - the state of this logical drive
      * @param size          - the size of this logical drive
      * @param raidLevel     - the raid level of this logical drive
      * @param array         - the array this logical drive belongs to 
      * @param controller    - the controller this logical drive belongs to
      */
    public StorageLogicalDrive(int  id,
                               int  state,
                               long size,
                               int  raidLevel,
                               int  array,
                               int  controller)
    {
        this.id = id;
        this.state = state;
        this.size = size;
        this.raidLevel = raidLevel;
        this.array = array;
        this.controller = controller;
    }

    /** Returns the Logical Drive Id
      * @return the Logical Drive Id
      */
    public final int getId()
    {
        return id;
    }
    
    /** Sets the Logical Drive Id
      * @param the id to set. 
      */
    public final void setId(int id)
    {
        this.id = id;
    }

    /** Returns the logical drive state
      * @return the logical drive state
      */
    public final int getState()
    {
        return state;
    }

    /** Returns the size of this logical drive
      * @return the size of this logical drive
      */
    public final long getSize()
    {
        return size;
    }

    /** Returns the Raid level
      * @return the Raid level
      */   
    public final int getRaidLevel()
    {
        return raidLevel;
    }

    /** Gets the the Logical Drive array
      * @return the Logical Drive array
      */   
    public final int getArray()
    {
        return array;
    }

    /** Sets the the Logical Drive array
      * @param the Logical Drive array
      */   
    public final void setArray(int array)
    {
        this.array = array;
    }
    
    /** Gets the the Logical Drive controller
      * @return the Logical Drive controller
      */   
    public final int getController()
    {
        return controller;
    }

    /** Sets the the Logical Drive controller
      * @param the Logical Drive controller
      */   
    public final void setController(int controller)
    {
        this.controller = controller;
    }
    
    /** Changes the parent for this object.
      * @param object  The new parent Enclosure
      * @param addToParent  If true, this object is added to its new parent.
      */
    public void setParent(StorageLogicalDrives parent, boolean addToParent)
    {
        setParent(parent);

        //Add ourselves to our new parent.
        if (addToParent)
            parent.add(this);
    }

    /** Returns the data used by the display table model.
      * @return  The multidimensional array of value sets used to populate the console display table.
      */

    public Vector getDisplayDataSet()
    {
        Vector v = new Vector();

        v.addElement(new Object[] { new ToolTipString("infoLogicalDriveState"), new StatusString(getDisplayState(), getOverallStatus())});
        v.addElement(new Object[] { new ToolTipString("infoLogicalDriveData"), String.valueOf(getSize())});
        v.addElement(new Object[] { new ToolTipString("infoLogicalDriveRaid"), String.valueOf(getRaidLevel())});
        
        return v;
    } 

    /** Returns the displayable logical drive state.
      * @returns the displayable logical drive state
      */
    public String getDisplayState()
    {
        switch (getState())
        {
            case OFL:  
                return JCRMUtil.getNLSString("infoLogDriveOffline");
            case OKY:  
                return JCRMUtil.getNLSString("infoLogDriveOkay");        
            case CRT:
                return JCRMUtil.getNLSString("infoLogDriveCritical");    
            default:   
                return JCRMUtil.getNLSString("unknown"); 
        }
    }

    /** Returns the filename for the icon to be shown in the tree for this RaidObject.
      * @returns the filename for the icon to be shown in the tree for this RaidObject
      */
    public String getDisplayIconFilename()
    {
        switch (getOverallStatus())
        {
            case NORMAL:    return "s_logdr1.gif";
            case CRITICAL:  return "s_logdr2.gif";
            case FATAL:     return "s_logdr3.gif";
            default:        return "s_logdr1.gif";
        }
    }

    /** Updates the overall status for this logical drive. 
      * @param recurse  true if this call should propagate down the config tree
      */
    public void updateOverallStatus(boolean recurse)
    {
        setOverallStatus(NORMAL);

        if (state == OFL)
            setOverallStatus(FATAL);
        else if (state == CRT)
            setOverallStatus(CRITICAL);
    }

    /** Satisfy comparable interface.  This method controls ordering in the tree.
      * Ordering of LogicalDrive objects is based on logical drive ID.
      * @param object  The object to compare to this one
      * @return  0 for equality, <0 if this object comes first, >0 if the passed-in object comes first
      */ 
    public int compareTo(Object object) throws ClassCastException
    {
        if (!(object instanceof StorageLogicalDrive))
            throw new ClassCastException("Object of type " + object.getClass().getName() + 
                                         " may not be compared to " + this.getClass().getName());
        else
        {
            StorageLogicalDrive drive = (StorageLogicalDrive) object;
            return(getId() - drive.getId());
        }
    }

    /** Returns the data headers used by the display table model.
      * @return  The array of value set names used to populate the console display table.
      */

    public String[] getDisplayDataSetNames()
    {
        return headers;
    }

    /** Returns the string used to identify this object in the tree.
      * @return  the string used to identify this object in the tree
      */
    public String getDisplayName()
    {
        Object[] args = { String.valueOf(getId()), JCRMUtil.getDisplayUnitValue((int)getSize()), JCRMUtil.getDisplayUnits()};
        return JCRMUtil.makeNLSString("treeLogicalDrive", args);
    }

    /** Returns the string used to identify this object in the window title bar.
      * @return  the string used to identify this object in the window title bar
      */
    public String getWindowDisplayName()
    {
        Object[] args = { String.valueOf(getId()), String.valueOf(getId())};
        return(JCRMUtil.getNLSString("titleGUI") + " " + JCRMUtil.makeNLSString("titleLogicalDriveInfo", args));
    }

    /** Determine equivalence.
      * @param object  The object to compare to this one
      * @return  True if the objects are functionally equivalent, false otherwise.
      */
    public boolean equals(Object object)
    {
        // Do this first, since it's fast.
        if (this == object)
            return true;

        // We DON'T subclass java.lang.Object, so we must check for superclass equality.
        if (!super.equals(object))
            return false;

        // Finally, we can do the cast and compare relevant fields.
        StorageLogicalDrive drive = (StorageLogicalDrive) object;

        if (getId() != drive.getId())
            return false;

        if (getRaidLevel() != drive.getRaidLevel())
            return false;

        // Everything matched, so we're equal!
        return true;
    }

    /** Since we override equals(), we must override hashcode().
      * @return a hash value for this object
      */
    public int hashCode()
    {
        int result = 0;
        result ^= super.hashCode();
        result = PRIME*result + getId();
        result = PRIME*result + getRaidLevel();
        return result;
    }
}
