package com.ibm.sysmgt.raidmgr.dataproc.config.external;

// ---------------------------------------------------------------------
//  Copyright:    (C) International Business Machines Corporation, 1998
//
//  Project Des:  Java Common Raid Manager
//
//  Module Name:  StorageLogicalDrives.java
//
//  Description:
//
//  Revision(s):  Title - Description            Author              Date
//  ------------  ------------------------     ---------------    ----------
//  0.0.0         Creation                      Dean Kalman       02/11/02

import java.util.*;
import com.ibm.sysmgt.raidmgr.util.*;
import com.ibm.sysmgt.raidmgr.common.*;
import com.ibm.sysmgt.raidmgr.dataproc.util.*;
import com.ibm.sysmgt.raidmgr.dataproc.config.*;

/** This class is the container object at the StorageSubsystem level for LogicalDrive objects.  
  * The only allowable children of LogicalDrives are LogicalDrive objects, as enforced 
  * by the add() method in this class.  
  */
public class StorageLogicalDrives extends RaidObject 
{
   static final long serialVersionUID =  -7161419199380112738L;

   /** The display headers used by the display table model */
   private final String headers[] =
   {  
      JCRMUtil.getNLSString("infoLogDrives"), 
      JCRMUtil.getNLSString("infoLogDrivesArray"),
      JCRMUtil.getNLSString("infoLogDrivesSize"),
      JCRMUtil.getNLSString("infoLogDrivesRaid")
   };

   /** The StorageLogicalDrives constructor.
     */
   public StorageLogicalDrives()
   {
   }

   /** Returns the number of logical drives in this container.
     * @return the number of logical drives in this container
     */
   public int getLogicalDriveCount()
   {
      int count = 0;

      for (Enumeration e = enumerateChildren(); e.hasMoreElements();)
      {
         Object object = e.nextElement();
         if (object instanceof StorageLogicalDrive)
            count++;
      }

      return count;
   }

   /** Returns an enumeration of the logical drives in this container.
     * @return an enumeration of the logical drives in this container
     */
   public Enumeration enumerateLogicalDrives()
   {
       Vector drives = new Vector();

       for (Enumeration e = enumerateChildren(); e.hasMoreElements();)
       {
           Object object = e.nextElement();
           if (object instanceof StorageLogicalDrive)
               drives.addElement(object);
       }
       return drives.elements();
   }

   /** This method satisfies LogicalDriveCollectionIntf.  It returns a Vector of the
     * logical drive objects contained in this object.
     * @param filter  If null, all drives are returned.  If non-null, return drives passed by the filter.
     * @return the Vector of logical drive objects
     */
   public Vector getLogicalDriveCollection(StorageLogicalDriveFilterIntf filter)
   {
      Vector drives = new Vector();
      for (Enumeration e = enumerateLogicalDrives(); e.hasMoreElements();)
      {
         Object object = e.nextElement();
         if (object instanceof StorageLogicalDrive)
         {
            StorageLogicalDrive drive = (StorageLogicalDrive) object;
            if ((filter == null) || (filter.accept(drive)))
               drives.addElement(drive);
         }
      }
      return drives;
   }

   /** Updates the overall status for the logical drives. 
     * @param recurse  true if this call should propagate down the config tree
     */
   public void updateOverallStatus(boolean recurse)
   {
       // Recursively update the status of all children
       if (recurse)
       {
          for (Enumeration e = enumerateChildren(); e.hasMoreElements();)
             ((RaidObject) e.nextElement()).updateOverallStatus(recurse);
       }

       // No children
       if (getChildCount() == 0)
          setOverallStatus(NORMAL);

       // Check for all FATAL children condition
       else if (getChildrenByStatus(FATAL).size() == getChildCount())
          setOverallStatus(FATAL);

       // Check for all NORMAL children condition
       else if (getChildrenByStatus(NORMAL).size() == getChildCount())
          setOverallStatus(NORMAL);

       // In-between is FATAL
       else
          setOverallStatus(FATAL);
   }

   /** Changes the parent for this object.
     * @param object  The new parent StorageSubsystem
     * @param addToParent  If true, this object is added to its new parent.
     */
   public void setParent(StorageSubsystem parent, boolean addToParent)
   {
      setParent(parent);

      //Add ourselves to our new parent.
      if (addToParent)
         parent.add(this);
   }

   /** Changes the parent for this object.
     * @param object  The new parent StorageGroup
     * @param addToParent  If true, this object is added to its new parent.
     */
   public void setParent(StorageGroup parent, boolean addToParent)
   {
      setParent(parent);

      //Add ourselves to our new parent.
      if (addToParent)
         parent.add(this);
   }

   /** Returns the data used by the display table model.
     * @return  The multidimensional array of value sets used to populate the console display table.
     */
   public Vector getDisplayDataSet()
   {
      Vector v = new Vector();

      for (Enumeration children = getLogicalDriveCollection(null).elements(); children.hasMoreElements();)
      {
         // Children can only be LogicalDrive objects (including subclasses), so this cast is safe.
         StorageLogicalDrive logDrv = (StorageLogicalDrive) children.nextElement();
         v.addElement(new Object[]
                     {
                        new StatusString(String.valueOf(logDrv.getId() + 1), logDrv.getOverallStatus()),
                        String.valueOf(logDrv.getArray()),
                        String.valueOf(logDrv.getSize()), 
                        String.valueOf(logDrv.getRaidLevel())
                     });
      }

      return v;
   }

   /** Returns the data headers used by the display table model.
     * @return  The array of value set names used to populate the console display table.
     */
   public String[] getDisplayDataSetNames()
   {
      return headers;
   }

   /** Returns the string used to identify this object in the tree.
     * @return  the string used to identify this object in the tree
     */
   public String getDisplayName()
   {
      RaidObject parent = getParent();

      if (parent instanceof StorageGroup)
         return JCRMUtil.getNLSString("treeLogDrivesInArray") + ' ' + parent.getDisplayID();
      else
         return JCRMUtil.getNLSString("treeLogicalDrives");
   }

   /** Returns the string used to identify this object in the window title bar.
     * @return  the string used to identify this object in the window title bar
     */
   public String getWindowDisplayName()
   {
      return(JCRMUtil.getNLSString("titleGUI"));
   }

   /** Returns the filename for the icon to be shown in the tree for this RaidObject.
     * @returns the filename for the icon to be shown in the tree for this RaidObject
     */
   public String getDisplayIconFilename()
   {
      switch (getOverallStatus())
      {
         case NORMAL:    return "s_lgdrs1.gif";
         case CRITICAL:  return "s_lgdrs2.gif";
         case FATAL:     return "s_lgdrs3.gif";
         default:        return "s_lgdrs1.gif";
      }
   }

   /** Returns the XML representation of this object.  Can optionally include the standard
     * XML header string.  All children of this object are embedded as XML elements in the output.
     * @param includeHeader  whether to include the XML header
     * @param recurse  whether to embed this object's children
     * @return  the XML representation of this object
     */
   public String toXML(boolean includeHeader, boolean recurse)
   {
      StringBuffer xml = new StringBuffer();

      if (includeHeader)
         xml.append("<?xml version='1.0' encoding='UTF-8'?>").append(SEP);

      xml.append("<STORAGELOGICALDRIVES>").append(SEP);

      if (recurse)
         for (Enumeration e = enumerateChildren(); e.hasMoreElements();)
            xml.append(((RaidObject) e.nextElement()).toXML(false, true)).append(SEP);

      xml.append("</STORAGELOGICALDRIVES>").append(SEP);

      return xml.toString();
   }

   /** Only LogicalDrive objects may be added to LogicalDrives objects.
     * @param object  a LogicalDrive
     */
   public void add(StorageLogicalDrive object)
   {
      super.add(object);
   }

   /** Satisfy comparable interface.  This method controls ordering in the tree.
     * @param object  The object to compare to this one
     * @return  0 for equality, -1 if this object comes first, 1 if the passed-in object comes first
     */ 
   public int compareTo(Object object) throws ClassCastException
   {
       if (object instanceof StorageEnclosure)
           return 1;
       else if (object instanceof StorageArrays)
           return 1;
       else if (object instanceof StorageLogicalDrives)
           return 0;
       else if (object instanceof StorageHotSpareDrives)
           return -1;
       else if (object instanceof StorageDevices)
           return -1;
       else
           throw new ClassCastException("Object of type " + object.getClass().getName() + 
                                        " may not be compared to " + this.getClass().getName());
   }

   /** Determine equivalence.
     * @param object  The object to compare to this one
     * @return  True if the objects are functionally equivalent, false otherwise.
     */
   public boolean equals(Object object)
   {
      // Do this first, since it's fast.
      if (this == object)
         return true;

      // We DON'T subclass java.lang.Object, so we must check for superclass equality.
      if (!super.equals(object))
         return false;

      // Finally, we can do the cast and compare relevant fields.
      StorageLogicalDrives logDrvs = (StorageLogicalDrives) object;

      // not equal if either parent reference is null, but not both
      if ((getParent() == null) != (logDrvs.getParent() == null))
         return false;
           
      // not equal if parent is not null but equals fails    
      if ((getParent() != null) && !getParent().equals(logDrvs.getParent()))
         return false;

      // Everything matched, so we're equal!
      return true;
   }

   /** Since we override equals(), we must override hashcode().
     * @return a hash value for this object
     */
   public int hashCode()
   {
      int result = 0;
      result ^= super.hashCode();
      result = PRIME*result + ((getParent() == null) ? 0 : getParent().hashCode());
      return result;
   }
}
