package com.ibm.sysmgt.raidmgr.dataproc.config.external;

// ---------------------------------------------------------------------
//  Copyright:    (C) International Business Machines Corporation, 2002
//
//  Project Des:  Java Common Raid Manager
//
//  Module Name:  StorageSwitch.java
//
//  Description:
//
//  Revision(s):  Title - Description            Author          Date
//  ------------  ------------------------     -----------    ----------
//  0.0.0         Creation                      DAK            01/30/02

import java.util.*;
import com.ibm.sysmgt.raidmgr.util.*;
import com.ibm.sysmgt.raidmgr.dataproc.util.*;
import com.ibm.sysmgt.raidmgr.dataproc.config.*;
import com.ibm.sysmgt.raidmgr.mgtGUI.actions.raid.*;

/** The storage switch base class.
  */
public class StorageSwitch extends RaidObject 
{
    static final long serialVersionUID = -5850336705929071492L;
    public static final int SWITCH = 1;
    public static final int HUB    = 2;

    /** The display headers used by the display table model */
    private final String headers[] =
    {  
        JCRMUtil.getNLSString("infoSwitchName"), 
        JCRMUtil.getNLSString("infoSwitchValue"),
    };       

    private int               id;
    private int               type;
    private String            hostName;
    private String            ipAddress;

    /** The StorageEnclosure constructor.
      * @param id - the id of this switch/hub
      * @param int - the type, either SWITCH or HUB
      * @param String - the hostName of this switch ethernet port
      * @param String - the TCP/IP address of this switch ethernet port 
      */
    public StorageSwitch(int    id,
                         int    type,
                         String hostName,
                         String ipAddress)  
    {
        this.id = id;
        this.type = type;
        this.hostName = hostName;
        this.ipAddress = ipAddress;
    }

    /** Returns this switch's id.
      * @return this switch's id
      */
    public final int getId()
    {
        return id;
    }

    /** Returns this switch's type.
      * @return this switch's type
      */
    public final int getType()
    {
        return type;
    }
    
    /** Returns this switch's host name.
      * @return this switch's host name
      */
    public final String getHostName()
    {
        return hostName;
    }

    /** Returns this switch's TCP/IP address.
      * @return this switch's TCP/IP address
      */
    public final String getIpAddress()
    {
        return ipAddress;
    }
    
    /** Returns the data headers used by the display table model.
      * @return  The array of value set names used to populate the console display table.
      */
    public String[] getDisplayDataSetNames()
    {
        return headers;
    }

    /** Returns the string used to identify this object in the window title bar.
      * @return  the string used to identify this object in the window title bar
      */
    public String getWindowDisplayName()
    {
        Object[] args = { getDisplayID()};
        return(JCRMUtil.getNLSString("titleGUI") + " " + JCRMUtil.makeNLSString("titleSwitchInfo", args));
    }

    /** Returns the filename for the icon to be shown in the tree for this RaidObject.
      * @returns the filename for the icon to be shown in the tree for this RaidObject
      */
    public String getDisplayIconFilename()
    {
        return "s_switch.gif";
    }

    /** Returns the string used to identify this object in the tree.
      * @return  the string used to identify this object in the tree
      */
    public String getDisplayName()
    {
        Object[] args = {getHostName()};
        if (getType() == SWITCH)
            return JCRMUtil.makeNLSString("treeSwitch", args);
        else 
            return JCRMUtil.makeNLSString("treeHub", args);
    }

    /** Returns the data used by the display table model.
      * @return  The multidimensional array of value sets used to populate the console display table.
      */
    public Vector getDisplayDataSet()
    {
        Vector v = new Vector();

        v.addElement(new Object[] { new ToolTipString("infoSwitchId"), String.valueOf(getId())});
        v.addElement(new Object[] { new ToolTipString("infoSwitchHostName"), getHostName()});
        v.addElement(new Object[] { new ToolTipString("infoSwitchIpAddress"), getIpAddress()});

        return v;
    }

    /** Get the actions (in the form of AbstractRaidAction objects) supported
      * by this RaidObject.  Used for building the contextual Actions menus.
      * @return the Vector of actions
      */
    public Vector getActions()
    {
        Vector actions = new Vector();

        if (getType() == SWITCH)
            actions.addElement(new LaunchBrowserAction("actionManageSwitch", getIpAddress(), "SwitchManager.html"));
        else 
            actions.addElement(new LaunchBrowserAction("actionManageHub", getIpAddress(), "SwitchManager.html"));

        setActions(null);
        return actions;
    }
    
    /** Returns the XML representation of this object.  Can optionally include the standard
      * XML header string.  All children of this object are embedded as XML elements in the output.
      * @param includeHeader  whether to include the XML header
      * @param recurse  whether to embed this object's children
      * @return  the XML representation of this object
      */
    public String toXML(boolean includeHeader, boolean recurse)
    {
        StringBuffer xml = new StringBuffer();

        if (includeHeader)
            xml.append("<?xml version='1.0' encoding='UTF-8'?>").append(SEP);

        xml.append("<STORAGESWITCH type=\"" + getType() + "\">").append(SEP);

        if (recurse)
            for (Enumeration e = enumerateChildren(); e.hasMoreElements();)
                xml.append(((RaidObject) e.nextElement()).toXML(false, true)).append(SEP);

        xml.append("</STORAGESWITCH>").append(SEP);

        return xml.toString();
    }
    
    /** Determine equivalence.
      * @param object  The object to compare to this one
      * @return  True if the objects are functionally equivalent, false otherwise.
      */
    public boolean equals(Object object)
    {
        // Do this first, since it's fast.
        if (this == object)
            return true;

        // We DON'T subclass java.lang.Object, so we must check for superclass equality.
        if (!super.equals(object))
            return false;

        // Finally, we can do the cast and compare relevant fields.
        StorageSwitch storageSwitch = (StorageSwitch) object;

        if (getId() != storageSwitch.getId())
            return false;

        // Everything matched, so we're equal!
        return true;
    }

    /** Satisfy comparable interface.  This method controls ordering in the tree.
      * Ordering of PhysicalDevice object is based on deviceID.
      * @param object  The object to compare to this one
      * @return  0 for equality, <0 if this object comes first, >0 if the passed-in object comes first
      */ 
    public int compareTo(Object object) throws ClassCastException
    {
        if (object instanceof StorageSwitch)
        {
            StorageSwitch storageSwitch = (StorageSwitch) object;
            if (storageSwitch.getType() == getType())
                return(getId() - storageSwitch.getId());
            else
                if (getType() == HUB)
                    return 1;
                else
                    return -1;
        }
        else if (object instanceof StorageSubsystem)
            return 1;
        else
            throw new ClassCastException("Object of type " + object.getClass().getName() + 
                                         " may not be compared to " + this.getClass().getName());
    }
    
    /** Since we override equals(), we must override hashcode().
      * @return a hash value for this object
      */
    public int hashCode()
    {
        int result = 0;
        result ^= super.hashCode();
        result = PRIME*result + getId() + getType();
        return result;
    }
}

