// ---------------------------------------------------------------------
//  Copyright:    (C) Adaptec Inc., 2002
//
//  Project Des:  Java Common Raid Manager
//
//  Module Name:  IrocAdapter.java
//
//  Description:
//
//  Revision(s):  Title - Description              Author          Date
//  ------------  ------------------------     --------------   ----------
//  0.0.0         Created for IROC             Clinton Knight    08/05/02

package com.ibm.sysmgt.raidmgr.dataproc.config.iroc;

import java.util.*;
import java.io.*;
import com.ibm.sysmgt.raidmgr.common.*;
import com.ibm.sysmgt.raidmgr.util.*;
import com.ibm.sysmgt.storage.api.*;
import com.ibm.sysmgt.raidmgr.dataproc.config.*;
import com.ibm.sysmgt.raidmgr.dataproc.util.*;
import com.ibm.sysmgt.raidmgr.mgtGUI.actions.*;
import com.ibm.sysmgt.raidmgr.mgtGUI.actions.base.*;
import com.ibm.sysmgt.raidmgr.mgtGUI.actions.raid.*;
import com.sun.wbem.cim.*;

public class IrocAdapter extends Adapter implements Constants
{
   static final long serialVersionUID = -1408254653532714722L;

   public static final String CIM_CREATION_CLASS_NAME             = "IBMPSG_IROCController";
   public static final String CIM_SNMP_CREATION_CLASS_NAME        = "IBMPSG_SNMPRAIDController";
   public static final String CIM_BIOS_CREATION_CLASS_NAME        = "IBMPSG_IROCBIOS";
   public static final String CIM_FIRMWARE_CREATION_CLASS_NAME    = "IBMPSG_IROCFirmware";
   public static final String CIM_DRIVER_CREATION_CLASS_NAME      = "IBMPSG_IROCDriver";

   private boolean             unattended;
   private String              adapterTypeKey;
   private IrocAdapterFeatures features;
   private InitiatorIDs        initiatorIDs;
   private Progress            progress;

   private String              displayName = null;

   /** Constructor for IrocAdapter */
   public IrocAdapter(RaidSystem         systemIN,
                      int                adapterTypeIN,
                      int                adapterIDIN,
                      int                adapterStatusIN,
                      String             biosVersionIN,
                      String             firmwareVersionIN,
                      String             driverVersionIN,
                      int                physicalSlotIN,
                      boolean            unattendedIN,
                      IrocAdapterFeatures featuresIN,
                      InitiatorIDs       initiatorIDsIN,
                      Progress           progressIN,
                      AdapterLimits      limitsIN )
   {
      super(systemIN, adapterTypeIN, adapterIDIN, adapterStatusIN, biosVersionIN, 
            firmwareVersionIN, driverVersionIN, physicalSlotIN, limitsIN);

      unattended   = unattendedIN;
      initiatorIDs = initiatorIDsIN;
      progress     = progressIN;
      features     = featuresIN;

      switch (adapterTypeIN)
      {
         case IROC:    adapterTypeKey = "infoControllerIROC";            break;
         default:      adapterTypeKey = "infoControllerAdaptecUnknown";  break;
      }

      // The children of this object are always present and valid (aggregates), but
      // they are only added as children to this object if this adapter is WORKING.
      // If this adapter is FAILED, we don't want these nodes in the tree.
      setArraysContainer(new IrocArrays(this));
      setLogicalDrivesContainer(new IrocLogicalDrives(this));
      setHotSpareDrivesContainer(new IrocHotSpareDrives(this));
      setPhysicalDrivesContainer(new IrocPhysicalDrives(this));
   }

   /** Returns the NLS key corresponding to the adapter type name.
     * @return the type key
     */
   public final String getAdapterTypeKey()
   {
      return adapterTypeKey;
   }

   /** Returns whether this adapter supports the specified feature.
     * @param feature The feature to check for
     * @return true if this adapter supports the specified feature
     */
   public boolean supports(int feature)
   {
      switch (feature)
      {
         case USER_DEFINED_LD_SIZE:            return false;
         case RESET_FACTORY_DEFAULTS:          return false;
         case ONLINE_CONFIGURATION:            return true;
         case SCSI_BUS_RESCAN:                 return false;
         case BATTERY_CACHE:                   return false;
         case DATA_SCRUBBING:                  return true;
         case AUTO_SYNC:                       return true;
         case CLUSTERING:                      return false;
         case OPTIMAL_SCSI:                    return false;
         case Adapter.LOGICAL_DRIVE_MIGRATION: return false;
         case ADAPTIVE_READ_AHEAD:             return false;
         case RAID_0:                          return(features.raid0 > 0);
         case RAID_1:                          return(features.raid1 > 0);
         case RAID_1E:                         return false;
         case RAID_5:                          return false;
         case RAID_5E:                         return false;
         case IDENTIFY_DRIVE:                  return true;
         case FLASHCOPY:                       return false;
         case LARGE_STRIPES_ON_LARGE_ARRAYS:   return false;
         case FIRST_FAILURE_DATA_CAPTURE:      return false;
         case BATTERY_BACKED_CACHE:            return false;
         case HIERARCHICAL_LOGICAL_DRIVES:     return true;
         case RAID_00:                         return false;
         case RAID_10:                         return(features.raid10 > 0); 
         case RAID_1E0:                        return false; 
         case RAID_50:                         return false; 
         case ULTRA_160_SCSI:                  return(features.ultra160scsi > 0);
         case ULTRA_320_SCSI:                  return(features.ultra320scsi > 0);
         case REAL_TIME_STAMP:                 return false;
         case HARD_DRIVE_FRU:                  return false;
         case SUBSYSTEM_STATISTICS:            return false;
         case RAID_5EE:                        return false;
         default:                              return false;
      }
   }

   /** Returns whether the specified mode of operation is enabled on this adapter.
     * The modes are defined by constants in this class.
     * @param feature  The operational mode to check for
     * @return true if this adapter mode is currently enabled
     */
   public boolean hasEnabled(int feature)
   {
      switch (feature)
      {
         case HOT_SWAP_REBUILD:  return true;
         case DATA_SCRUBBING:    return true;
         case AUTO_SYNC:         return true;
         case UNATTENDED:        return unattended;
         default:                return false;
      }
   }

   /** Sets the specified mode of operation is enabled on this adapter.
     * The modes are defined by constants in this class.
     * @param feature  The operational mode to set
     * @param b  if this adapter mode is to be enabled
     */
   protected void setEnabled(int feature, boolean b)
   {
      switch (feature)
      {
         case UNATTENDED:  unattended = b;  break;
         default:                           break;
      }
   }

   /** Returns an InitiatorIDs object containing the SCSI initiator ID values
     * used by this adapter.
     * @return the InitiatorIDs object
     */
   public final InitiatorIDs getInitiatorIDs()
   {
      return initiatorIDs;
   }

   /** Returns the maximum number of drives allowed per array.
     * @return the maximum number of drives allowed per array
     */
   public int getMaxPhysicalDrivesPerArray()
   {
      return(getLimit(MAX_DEVICES_PER_ARRAY));
   }

   /** Returns the minimum number of drives allowed per array.
     * @return the minimum number of drives allowed per array
     */
   public int getMinPhysicalDrivesPerArray()
   {
      return 1;
   }

   /** Returns the maximum number of logical drives allowed per array.
     * @return the maximum number of logical drives allowed per array
     */
   public int getMaxLogicalDrivesPerArray()
   {
      return 1;
   }

   /** Returns the maximum logical drive size allowed (in megabytes).
     * The max value is 2**32 * 512 = 2TB.
     * @return the maximum logical drive size allowed (in megabytes)
     */
   public int getMaxLogicalDriveSize()
   {
      return ServeRAID_2TB_megabytes - 1;
   }

   /** Returns the maximum arrays allowed on this controller.
     * @return the maximum number of arrays allowed on this controller
     */
   public int getMaxArrays()
   {
      return(getLimit(MAX_ARRAYS));
   }

   /** Returns the maximum number of logical drives supported.
     * @return the maximum number of logical drives supported
     */
   public int getMaxLogicalDrives() 
   {
      return(getLimit(MAX_VIRT_DRIVES));
   }

   /** Returns the maximum number of hot spare drives supported.
     * @return the maximum number of hot spare drives supported
     */
   public int getMaxHotSpares()
   {
      return 0; 
   }

   /** Returns the maximum number of hot spare drives that may be created now.
     * @return the maximum number of hot spare drives that may be created now
     */
   public int getMaxAdditionalHotSpares()
   {
      return Math.min(getReadyDriveCount(), getMaxHotSpares() - getHotSpareDriveCount());
   }

   /** Returns true if another logical drive may be created.
     * @return true if another logical drive may be created
     */
   public boolean hasLogicalDriveSlotAvailable()
   {
      return(getLogicalDriveCount() < getMaxLogicalDrives());
   }

   /** Returns true if another hierarchical logical drive may be created.
     * @return true if another hierarchical logical drive may be created
     */
   public boolean hasHLogicalDriveSlotAvailable()
   {
      return((getMaxLogicalDrives() - getLogicalDriveCount()) >= 2);
   }

   /** Returns true if the controller is configured.  'Configured' means that
     * the controller has logical drives defined.  Subclasses may perform additional
     * checks.
     * @return true if the controller is configured
     */
   public boolean isConfigured()
   {
      return(getLogicalDriveCount() > 0);
   }

   /** Returns the state of the logical drive with the specified ID
     * @param ID  the ID of the logical drive being sought
     * @param isSLDIndex  true if this logical drive is a sub-logical drive, false otherwise (not used)
     * @param the state of the logical drive in question, or FRE if non-existent
     */
   public int getLogicalDriveState(int ID, boolean isSLDIndex)
   {
      LogicalDrive drive = getBasicLogicalDrive(ID);
      return((drive != null) ? drive.getState() : FRE);
   }

   /** Returns a reference to the progress object for this adapter
     * @return a reference to the progress object for this adapter
     */
   public final Progress getProgress()
   {
      return progress;
   }

   /** Sets the progress object for this adapter
     * @param a reference to the progress object for this adapter
     */
   public final void setProgress(Progress p)
   {
      progress = p;
   }

   /** Returns the action to be displayed in the event that all actions are blocked.
     * @return the action to be displayed in the event that all actions are blocked
     */
   public AbstractRaidAction getBlockerAction()
   {
      if (progress == null)
         return new NullAction();

      else
         return new ProgressBlockerAction(progress);
   }

   /**
     * Retrieve the HLD Id for the given SLD Id
     * @param     The Sub-Logical drive Id
     * @return    The Heirarchical Logical Drive Id associated with the given SLD
     *            -1 if the HLD Id could not be determined
     */
   public int getHLogDrvId(int sldId)
   {
      return -1;
   }

   /** Get the actions (in the form of AbstractRaidAction objects) supported
     * by this RaidObject.  Used for building the contextual Actions menus.
     * @return the Vector of actions
     */
   public Vector getActions()
   {
      if (super.getActions() == null)
      {
         Vector actions = new Vector();
         actions.addElement(new HelpNowAction(this));
         actions.addElement(new NullAction());
         actions.addElement(new ConfigAdapterAction(this));
         actions.addElement(new DeleteArraysAction(this));
         actions.addElement(new NullAction());
         actions.addElement(new ToggleUnattendedModeAction(this));
         actions.addElement(new NullAction());
         actions.addElement(new IdentifyDriveAction(this, IdentifyDriveAction.ID_ALL_PHY_DRVS));
         setActions(actions);
      }
      return super.getActions();
   }

   /** Returns the data used by the display table model.
     * @return  The multidimensional array of value sets used to populate the console display table.
     */
   public Vector getDisplayDataSet()
   {
      // copy data points from the Adapter superclass
      Vector v = new Vector();

      if (getStatus() == FAILED)
      {
         v.addElement(new Object[] { new ToolTipString("infoControllerNumber"), getDisplayID()});
         v.addElement(new Object[] { new ToolTipString("infoControllerStatus"), new StatusString(JCRMUtil.getNLSString("infoControllerNotWork"), FATAL)});
      }
      else
      {
         v.addElement(new Object[] { new ToolTipString("infoControllerTypeIROC"), JCRMUtil.getNLSString(getAdapterTypeKey())});

         if (!getBIOSVersion().equals(""))
            v.addElement(new Object[] { new ToolTipString("infoControllerBios"), getBIOSVersion()});

         if (!getFirmwareVersion().equals(""))
            v.addElement(new Object[] { new ToolTipString("infoControllerFirmware"), getFirmwareVersion()});

         if (!getDriverVersion().equals(""))
            v.addElement(new Object[] { new ToolTipString("infoControllerDeviceDriver"), getDriverVersion()});

         v.addElement(new Object[] { new ToolTipString("infoControllerHSR"), 
                         hasEnabled(HOT_SWAP_REBUILD) ? JCRMUtil.getNLSString("enabled") : JCRMUtil.getNLSString("disabled")});

         if (supports(DATA_SCRUBBING))
            v.addElement(new Object[] { new ToolTipString("infoControllerDataScrub"), 
                            hasEnabled(DATA_SCRUBBING) ? JCRMUtil.getNLSString("enabled") : JCRMUtil.getNLSString("disabled")});

         if (supports(AUTO_SYNC))
            v.addElement(new Object[] { new ToolTipString("infoControllerAutoSync"), 
                            hasEnabled(AUTO_SYNC) ? JCRMUtil.getNLSString("enabled") : JCRMUtil.getNLSString("disabled")});

         v.addElement(new Object[] { new ToolTipString("infoControllerUnattend"), 
                         hasEnabled(UNATTENDED) ? JCRMUtil.getNLSString("enabled") : JCRMUtil.getNLSString("disabled")});

         v.addElement(new String[] { JCRMUtil.getNLSString("infoControllerArrays"), String.valueOf(getArrayCount())});

         v.addElement(new String[] { JCRMUtil.getNLSString("infoControllerLogDrives"), String.valueOf(getLogicalDriveCount())});

         v.addElement(new String[] { JCRMUtil.getNLSString("infoControllerHotSpares"), String.valueOf(getHotSpareDriveCount())});

         v.addElement(new String[] { JCRMUtil.getNLSString("infoControllerReadys"), String.valueOf(getReadyDriveCount())});
      }

      return v;
   }

   /** Returns the string used to identify this object in the tree.
     * @return  the string used to identify this object in the tree
     */
   public String getDisplayName()
   {
      if (displayName != null)
         return displayName;

      if (getStatus() == WORKING)
      {
         Object[] args = { getDisplayID()};
         if (!isConfigured())
            return JCRMUtil.makeNLSString( "treeControllerNotConfig", args);
         else
            return JCRMUtil.makeNLSString( "treeController", args );
      }
      else if (getStatus() == FAILED)
      {
         Object[] args = { getDisplayID()};
         return JCRMUtil.makeNLSString( "treeControllerNotRespond", args);
      }

      return "";
   }

   /** Sets the string used to identify this object in the tree.
     * @param the string used to identify this object in the tree
     */
   public final void setDisplayName(String displayName)
   {
      this.displayName = displayName;
   }

   /** Returns the shorter string used to identify this object in the status bar, etc.
     * @return  the shorter string used to identify this object in the status bar, etc
     */
   public String getShortDisplayName()
   {
      Object[] args = { getDisplayID()};
      return JCRMUtil.makeNLSString("treeController", args);
   }

   /** Returns the filename for the icon to be shown in the tree for this RaidObject.
     * @returns the filename for the icon to be shown in the tree for this RaidObject
     */
   public String getDisplayIconFilename()
   {
      switch (getOverallStatus())
      {
         case CRITICAL: return "s_adapt2.gif";
         case FATAL:    return "s_adapt3.gif";
         case NORMAL:
         default:       return "s_adapt1.gif";
      }
   }

   /** This method allows providers to display context-sensitive help.
     * Various software components can register their current context
     * with their associated help provider, so that when help is invoked,
     * the correct content is displayed.
     * @return String  the ID of the most contextually-appropriate help content
     */
   public String getHelpContext()
   {
      return "helpIROCAdapter";
   }

   /** Returns the XML representation of this object.  Can optionally include the standard
     * XML header string.  All children of this object are embedded as XML elements in the output.
     * @param includeHeader  whether to include the XML header
     * @param recurse  whether to embed this object's children
     * @return  the XML representation of this object
     */
   public String toXML(boolean includeHeader, boolean recurse)
   {
      StringBuffer xml = new StringBuffer();

      if (includeHeader)
         xml.append("<?xml version='1.0' encoding='UTF-8'?>").append(SEP);

      xml.append("<ADAPTER>").append(SEP);

      if (recurse)
         for (Enumeration e = enumerateChildren(); e.hasMoreElements();)
            xml.append(((RaidObject) e.nextElement()).toXML(false, true)).append(SEP);

      xml.append("</ADAPTER>").append(SEP);

      return xml.toString();
   }

   /** Returns a shallow clone of this object, sans children.
     * @return a shallow clone of this object, sans children
     */
   public RaidObject emptyClone()
   {
      IrocAdapter adapter = new IrocAdapter(getRaidSystem(), 
                                            getAdapterType(), 
                                            getID(), 
                                            getStatus(), 
                                            getBIOSVersion(), 
                                            getFirmwareVersion(),
                                            getDriverVersion(), 
                                            getPhysicalSlot(), 
                                            hasEnabled(UNATTENDED),
                                            features,
                                            getInitiatorIDs(), 
                                            getProgress(),
                                            getLimits());

      adapter.remove(getArraysContainer()); 
      adapter.remove(getLogicalDrivesContainer());
      adapter.remove(getHotSpareDrivesContainer());
      adapter.remove(getPhysicalDrivesContainer());
      return adapter;
   }

   /** Looks for changes in this RaidObject, relative to the specified object, that 
     * are unrelated to an object's identity.  For example, changing the type of an
     * adapter would cause two objects to fail the equals() test, but changing an
     * adapter's stripe unit size would not.  This method looks for such changes.
     * It always calls equals() first to ensure we don't compare dissimilar objects.
     * @param object  The RaidObject to be compared to this one
     * @return true if this RaidObject has reportable differences from the specified RaidObject, false otherwise.
     */
   public boolean hasChangesRelativeTo(RaidObject object)
   {
      // An object should only be compared to itself
      if (!object.equals(this))
         return false;

      // Do the cast and compare relevant fields.
      IrocAdapter adapter = (IrocAdapter) object;

      // Compare all relevant fields
      if (unattended != adapter.unattended)
         return true;

      // No changes found
      return false;
   }

   /** Override the toString method.  Useful primarily for debugging concrete classes.
     * @return  a String representing this RaidObject
     */
   public String toString()
   {
      return("Adapter " + getDisplayID() + " : type=" + JCRMUtil.getNLSString(adapterTypeKey) + " : slot=" + getPhysicalSlot());
   }

   public CIMInstance createCIMInstance(CIMInstance instance) throws CIMException
   {
      // Keys
      instance.setProperty("CreationClassName", new CIMValue(CIM_CREATION_CLASS_NAME));
      instance.setProperty("SystemCreationClassName", new CIMValue(com.ibm.sysmgt.raidmgr.cim.provider.CIMUtil.getSystemCreationClassName()));
      instance.setProperty("SystemName", new CIMValue(JCRMNet.getHostname()));
      instance.setProperty("DeviceID", new CIMValue(new String("IBM:ServeRAID:ControllerID:" + getAdapter().getDisplayID())));

      instance.setProperty("ControllerID", new CIMValue(new String(getDisplayID())));

      if (getStatus() == Constants.WORKING)
         instance.setProperty("Status", new CIMValue(new String(JCRMUtil.getNLSString("cimOK"))));
      else
         instance.setProperty("Status", new CIMValue(new String(JCRMUtil.getNLSString("cimNonRecover"))));

      instance.setProperty("UnattendedMode", new CIMValue(new Boolean(hasEnabled(UNATTENDED))));

      instance.setProperty("BIOSVersion", new CIMValue(getBIOSVersion()));
      instance.setProperty("FirmwareVersion", new CIMValue(getFirmwareVersion()));
      instance.setProperty("DriverVersion", new CIMValue(getDriverVersion()));

      instance.setProperty("Manufacturer", new CIMValue(new String(JCRMUtil.getNLSString("cimIROCManufacturer"))));
      instance.setProperty("Model", new CIMValue(new String(JCRMUtil.getNLSString(getAdapterTypeKey()))));

      instance.setProperty("Slot", new CIMValue(new UnsignedInt16(getPhysicalSlot())));

      instance.setProperty("ProtocolSupported", new CIMValue(new UnsignedInt16(9) /* SCSI Parallel Interface */));
      instance.setProperty("MaxNumberControlled", new CIMValue(new UnsignedInt32(getChannelCount() * (getLimit(MAX_SCSI_ID) + 1))));

      String initiatorIDs = new String();

      for (Enumeration e = enumerateChannels(); e.hasMoreElements();)
      {
         SCSIChannel channel = (SCSIChannel)e.nextElement();
         initiatorIDs = initiatorIDs + channel.getInitiatorID();

         if (e.hasMoreElements())
            initiatorIDs = initiatorIDs + ",";
      }

      instance.setProperty("InitiatorIDs", new CIMValue(initiatorIDs));

      return instance;
   }

   public CIMInstance createCIMSNMPInstance(CIMInstance instance) throws CIMException
   {
      // Keys
      instance.setProperty("ControllerID", new CIMValue(new UnsignedInt32(Integer.parseInt(getDisplayID()))));

      instance.setProperty("BIOSVersion", new CIMValue(getBIOSVersion()));
      instance.setProperty("FirmwareVersion", new CIMValue(getFirmwareVersion()));
      instance.setProperty("VendorName", new CIMValue(new String(JCRMUtil.getNLSString("cimIROCManufacturer"))));
      instance.setProperty("Model", new CIMValue(new String(JCRMUtil.getNLSString(getAdapterTypeKey()))));
      instance.setProperty("SlotNumber", new CIMValue(new UnsignedInt32(getPhysicalSlot())));

      if (getStatus() == Constants.WORKING)
         instance.setProperty("GeneralStatus", new CIMValue(new UnsignedInt16(1)));
      else
         instance.setProperty("GeneralStatus", new CIMValue(new UnsignedInt16(2)));

      instance.setProperty("MaxChannels", new CIMValue(new UnsignedInt32(getChannelCount())));                                            
      instance.setProperty("MaxLogicalDrives", new CIMValue(new UnsignedInt32(getLimit(MAX_VIRT_DRIVES))));
      instance.setProperty("MaxPhysicalDevices", new CIMValue(new UnsignedInt32(getLimit(MAX_SCSI_ID) * getChannelCount())));

      // ******************************************************************************************
      // Since the CIM providers do a "getConfigAll" we need to filter out the place holder data
      // ******************************************************************************************

      int activeChannels[] = new int[getChannelCount()];

      for (int i = 0; i < activeChannels.length; i++)
         activeChannels[i] = 0;

      int numChannels = 0;
      int numLogicalDrives = 0;
      int numPhysicalDrives = 0;

      // Figure out how many channels have devices attached
      // Also use this iteration to count the number of devices attached to the controller
      for (Enumeration e = getPhysicalDeviceCollection(null).elements(); e.hasMoreElements();)
      {
         PhysicalDevice d = (PhysicalDevice)e.nextElement();
         if (d.getState() != Constants.EMP)
         {
            activeChannels[d.getChannel().getID()] = activeChannels[d.getChannel().getID()] + 1;
            numPhysicalDrives++;
         }
      }
      for (int i = 0; i < activeChannels.length; i++)
      {
         if (activeChannels[i] > 0)
            numChannels++;
      }

      // Figure out how many configured logical drives there are
      for (Enumeration e = getLogicalDriveCollection(null).elements(); e.hasMoreElements();)
      {
         LogicalDrive ld = (LogicalDrive)e.nextElement();
         if (ld.getState() != Constants.FRE)
            numLogicalDrives++;
      }

      instance.setProperty("NumChannels", new CIMValue(new UnsignedInt32(numChannels)));
      instance.setProperty("NumLogicalDrives", new CIMValue(new UnsignedInt32(numLogicalDrives)));
      instance.setProperty("NumPhysicalDevices", new CIMValue(new UnsignedInt32(numPhysicalDrives)));

      return instance;
   }

   public CIMInstance createCIMBIOSInstance(CIMInstance instance) throws CIMException
   {
      // keys
      instance.setProperty("Name", new CIMValue(new String(JCRMUtil.getNLSString("cimIROCBIOS") + " " + getDisplayID())));
      instance.setProperty("SoftwareElementState", new CIMValue(new UnsignedInt16(3) /* Running */));
      instance.setProperty("SoftwareElementID", new CIMValue(new String(getDisplayID())));
      instance.setProperty("TargetOperatingSystem", new CIMValue(new UnsignedInt16(63) /* Not Applicable */));
      instance.setProperty("Version", new CIMValue(getBIOSVersion()));

      instance.setProperty("Manufacturer", new CIMValue(JCRMUtil.getNLSString("cimIROCManufacturer")));

      String buildNumber = "";
      try
      {
         StringTokenizer st = new StringTokenizer(getBIOSVersion(), ".");
         st.nextToken();
         st.nextToken();         
         buildNumber = st.nextToken();
      }
      catch (NoSuchElementException ex)
      {
      }

      instance.setProperty("BuildNumber", new CIMValue(buildNumber));
      instance.setProperty("Description", new CIMValue(JCRMUtil.getNLSString("cimIROCBIOSDesc")));
      //instance.setProperty("ReleaseDate", new CIMValue("IBM"));
      instance.setProperty("PrimaryBIOS", new CIMValue(new Boolean(false)));

      return instance;
   }

   public CIMInstance createCIMFirmwareInstance(CIMInstance instance) throws CIMException
   {  
      // keys
      instance.setProperty("Name", new CIMValue(new String(JCRMUtil.getNLSString("cimIROCFirmware") + " " + getDisplayID())));
      instance.setProperty("SoftwareElementState", new CIMValue(new UnsignedInt16(3) /* Running */));
      instance.setProperty("SoftwareElementID", new CIMValue(new String(getDisplayID())));
      instance.setProperty("TargetOperatingSystem", new CIMValue(new UnsignedInt16(63) /* Not Applicable */));
      instance.setProperty("Version", new CIMValue(getFirmwareVersion()));

      instance.setProperty("Manufacturer", new CIMValue(JCRMUtil.getNLSString("cimIROCManufacturer")));

      String buildNumber = "";
      try
      {
         StringTokenizer st = new StringTokenizer(getFirmwareVersion(), ".");
         st.nextToken();
         st.nextToken();
         buildNumber = st.nextToken();
      }
      catch (NoSuchElementException ex)
      {
      }

      instance.setProperty("BuildNumber", new CIMValue(buildNumber));
      instance.setProperty("Description", new CIMValue(JCRMUtil.getNLSString("cimIROCFirmwareDesc")));
      //instance.setProperty("ReleaseDate", new CIMValue("IBM"));
      instance.setProperty("PrimaryBIOS", new CIMValue(new Boolean(false)));

      return instance;
   }

   public CIMInstance createCIMDriverInstance(CIMInstance instance) throws CIMException
   {
      // keys
      instance.setProperty("Name", new CIMValue(new String(JCRMUtil.getNLSString("cimIROCDriver") + " " + getDisplayID())));
      instance.setProperty("SoftwareElementState", new CIMValue(new UnsignedInt16(3) /* Running */));
      instance.setProperty("SoftwareElementID", new CIMValue(new String(getDisplayID())));
      instance.setProperty("TargetOperatingSystem", new CIMValue(new UnsignedInt16(63) /* Not Applicable */));
      instance.setProperty("Version", new CIMValue(getDriverVersion()));

      instance.setProperty("Manufacturer", new CIMValue(JCRMUtil.getNLSString("cimIROCManufacturer")));

      String buildNumber = "";
      try
      {
         StringTokenizer st = new StringTokenizer(getDriverVersion(), ".");
         st.nextToken();
         st.nextToken();
         buildNumber = st.nextToken();
      }
      catch (NoSuchElementException ex)
      {
      }

      instance.setProperty("BuildNumber", new CIMValue(buildNumber));
      instance.setProperty("Description", new CIMValue(JCRMUtil.getNLSString("cimIROCDriverDesc")));

      return instance;
   }

   public CIMInstance createAssociatedFirmwareAssociation(CIMInstance instance) throws CIMException
   {
      // Antecedent
      CIMObjectPath op2 = new CIMObjectPath(CIM_FIRMWARE_CREATION_CLASS_NAME);

      op2.addKey("Name", new CIMValue(new String(JCRMUtil.getNLSString("cimIROCFirmware") + " " + getDisplayID())));
      op2.addKey("SoftwareElementState", new CIMValue(new UnsignedInt16(3) /* Running */));
      op2.addKey("SoftwareElementID", new CIMValue(new String(getDisplayID())));
      op2.addKey("TargetOperatingSystem", new CIMValue(new UnsignedInt16(63) /* Not Applicable */));
      op2.addKey("Version", new CIMValue(getFirmwareVersion()));

      instance.setProperty("Antecedent", new CIMValue(op2));


      // Dependent
      instance.setProperty("Dependent", new CIMValue(getCIMObjectPath()));

      return instance;
   }

   public CIMInstance createAssociatedBIOSAssociation(CIMInstance instance) throws CIMException
   {
      // Antecedent
      CIMObjectPath op2 = new CIMObjectPath(CIM_BIOS_CREATION_CLASS_NAME);

      op2.addKey("Name", new CIMValue(new String(JCRMUtil.getNLSString("cimIROCBIOS") + " " + getDisplayID())));
      op2.addKey("SoftwareElementState", new CIMValue(new UnsignedInt16(3) /* Running */));
      op2.addKey("SoftwareElementID", new CIMValue(new String(getDisplayID())));
      op2.addKey("TargetOperatingSystem", new CIMValue(new UnsignedInt16(63) /* Not Applicable */));
      op2.addKey("Version", new CIMValue(getBIOSVersion()));

      instance.setProperty("Antecedent", new CIMValue(op2));


      // Dependent
      instance.setProperty("Dependent", new CIMValue(getCIMObjectPath()));

      return instance;
   }

   public CIMInstance createAssociatedDriverAssociation(CIMInstance instance) throws CIMException
   {
      // Antecedent
      CIMObjectPath op2 = new CIMObjectPath(CIM_DRIVER_CREATION_CLASS_NAME);

      op2.addKey("Name", new CIMValue(new String(JCRMUtil.getNLSString("cimIROCDriver") + " " + getDisplayID())));
      op2.addKey("SoftwareElementState", new CIMValue(new UnsignedInt16(3) /* Running */));
      op2.addKey("SoftwareElementID", new CIMValue(new String(getDisplayID())));
      op2.addKey("TargetOperatingSystem", new CIMValue(new UnsignedInt16(63) /* Not Applicable */));
      op2.addKey("Version", new CIMValue(getDriverVersion()));

      instance.setProperty("Antecedent", new CIMValue(op2));


      // Dependent
      instance.setProperty("Dependent", new CIMValue(getCIMObjectPath()));

      return instance;
   }

   public CIMObjectPath getCIMObjectPath()
   {
      CIMObjectPath op = new CIMObjectPath(CIM_CREATION_CLASS_NAME);

      op.addKey("CreationClassName", new CIMValue(CIM_CREATION_CLASS_NAME));
      op.addKey("SystemCreationClassName", new CIMValue(com.ibm.sysmgt.raidmgr.cim.provider.CIMUtil.getSystemCreationClassName()));
      op.addKey("SystemName", new CIMValue(JCRMNet.getHostname()));
      op.addKey("DeviceID", new CIMValue(new String("IBM:ServeRAID:ControllerID:" + getAdapter().getDisplayID())));

      return op;
   }
}

